AC_DEFUN([DUMPI_TEST_MPI_OPS],
  [
    dnl Test whether MPI operations are defined properly
    dnl The MPI implementation on Hera at LLNL fails to define some of these
    AC_LANG_PUSH(C)

    AH_TEMPLATE([DUMPI_HAVE_MPI_OP_NULL], [defined if MPI_OP_NULL is available])
    AC_MSG_CHECKING([whether the operation MPI_OP_NULL is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Op op = MPI_OP_NULL;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_OP_NULL, [1])
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_ERROR([no])
      ]
    )

    AH_TEMPLATE([DUMPI_OP_MPI_MAX],
                [MPI_MAX if MPI_MAX is defined, else MPI_OP_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_MAX], [defined if MPI_MAX is available])
    AC_MSG_CHECKING([whether the operation MPI_MAX is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Op op = MPI_MAX;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_MAX, [1])
        AC_DEFINE(DUMPI_OP_MPI_MAX, [MPI_MAX])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_OP_MPI_MAX, [MPI_OP_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_OP_MPI_MIN],
                [MPI_MIN if MPI_MIN is defined, else MPI_OP_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_MIN], [defined if MPI_MIN is available])
    AC_MSG_CHECKING([whether the operation MPI_MIN is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Op op = MPI_MIN;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_MIN, [1])
        AC_DEFINE(DUMPI_OP_MPI_MIN, [MPI_MIN])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_OP_MPI_MIN, [MPI_OP_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_OP_MPI_SUM],
                [MPI_SUM if MPI_SUM is defined, else MPI_OP_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_SUM], [defined if MPI_SUM is available])
    AC_MSG_CHECKING([whether the operation MPI_SUM is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Op op = MPI_SUM;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_SUM, [1])
        AC_DEFINE(DUMPI_OP_MPI_SUM, [MPI_SUM])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_OP_MPI_SUM, [MPI_OP_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_OP_MPI_PROD],
                [MPI_PROD if MPI_PROD is defined, else MPI_OP_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_PROD], [defined if MPI_PROD is available])
    AC_MSG_CHECKING([whether the operation MPI_PROD is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Op op = MPI_PROD;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_PROD, [1])
        AC_DEFINE(DUMPI_OP_MPI_PROD, [MPI_PROD])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_OP_MPI_PROD, [MPI_OP_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_OP_MPI_LAND],
                [MPI_LAND if MPI_LAND is defined, else MPI_OP_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_LAND], [defined if MPI_LAND is available])
    AC_MSG_CHECKING([whether the operation MPI_LAND is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Op op = MPI_LAND;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_LAND, [1])
        AC_DEFINE(DUMPI_OP_MPI_LAND, [MPI_LAND])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_OP_MPI_LAND, [MPI_OP_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_OP_MPI_BAND],
                [MPI_BAND if MPI_BAND is defined, else MPI_OP_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_BAND], [defined if MPI_BAND is available])
    AC_MSG_CHECKING([whether the operation MPI_BAND is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Op op = MPI_BAND;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_BAND, [1])
        AC_DEFINE(DUMPI_OP_MPI_BAND, [MPI_BAND])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_OP_MPI_BAND, [MPI_OP_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_OP_MPI_LOR],
                [MPI_LOR if MPI_LOR is defined, else MPI_OP_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_LOR], [defined if MPI_LOR is available])
    AC_MSG_CHECKING([whether the operation MPI_LOR is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Op op = MPI_LOR;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_LOR, [1])
        AC_DEFINE(DUMPI_OP_MPI_LOR, [MPI_LOR])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_OP_MPI_LOR, [MPI_OP_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_OP_MPI_BOR],
                [MPI_BOR if MPI_BOR is defined, else MPI_OP_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_BOR], [defined if MPI_BOR is available])
    AC_MSG_CHECKING([whether the operation MPI_BOR is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Op op = MPI_BOR;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_BOR, [1])
        AC_DEFINE(DUMPI_OP_MPI_BOR, [MPI_BOR])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_OP_MPI_BOR, [MPI_OP_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_OP_MPI_LXOR],
                [MPI_LXOR if MPI_LXOR is defined, else MPI_OP_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_LXOR], [defined if MPI_LXOR is available])
    AC_MSG_CHECKING([whether the operation MPI_LXOR is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Op op = MPI_LXOR;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_LXOR, [1])
        AC_DEFINE(DUMPI_OP_MPI_LXOR, [MPI_LXOR])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_OP_MPI_LXOR, [MPI_OP_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_OP_MPI_BXOR],
                [MPI_BXOR if MPI_BXOR is defined, else MPI_OP_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_BXOR], [defined if MPI_BXOR is available])
    AC_MSG_CHECKING([whether the operation MPI_BXOR is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Op op = MPI_BXOR;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_BXOR, [1])
        AC_DEFINE(DUMPI_OP_MPI_BXOR, [MPI_BXOR])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_OP_MPI_BXOR, [MPI_OP_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_OP_MPI_MINLOC],
                [MPI_MINLOC if MPI_MINLOC is defined, else MPI_OP_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_MINLOC], [defined if MPI_MINLOC is available])
    AC_MSG_CHECKING([whether the operation MPI_MINLOC is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Op op = MPI_MINLOC;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_MINLOC, [1])
        AC_DEFINE(DUMPI_OP_MPI_MINLOC, [MPI_MINLOC])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_OP_MPI_MINLOC, [MPI_OP_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_OP_MPI_MAXLOC],
                [MPI_MAXLOC if MPI_MAXLOC is defined, else MPI_OP_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_MAXLOC], [defined if MPI_MAXLOC is available])
    AC_MSG_CHECKING([whether the operation MPI_MAXLOC is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Op op = MPI_MAXLOC;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_MAXLOC, [1])
        AC_DEFINE(DUMPI_OP_MPI_MAXLOC, [MPI_MAXLOC])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_OP_MPI_MAXLOC, [MPI_OP_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_OP_MPI_REPLACE],
                [MPI_REPLACE if MPI_REPLACE is defined, else MPI_OP_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_REPLACE], [defined if MPI_REPLACE is available])
    AC_MSG_CHECKING([whether the operation MPI_REPLACE is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Op op = MPI_REPLACE;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_REPLACE, [1])
        AC_DEFINE(DUMPI_OP_MPI_REPLACE, [MPI_REPLACE])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_OP_MPI_REPLACE, [MPI_OP_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AC_LANG_POP
  ]
)
