AC_DEFUN([DUMPI_TEST_MPI_TYPES],
  [
    dnl Test whether MPI datatypes are defined properly
    dnl The MPI implementation on Hera at LLNL fails to define some of these
    AC_LANG_PUSH(C)

    AH_TEMPLATE([DUMPI_HAVE_MPI_DATATYPE_NULL], [defined if MPI_DATATYPE_NULL is available])
    AC_MSG_CHECKING([whether the datatype MPI_DATATYPE_NULL is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_DATATYPE_NULL;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_DATATYPE_NULL, [1])
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_ERROR([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_CHAR],
		[MPI_CHAR if MPI_CHAR is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_CHAR], [defined if MPI_CHAR is available])
    AC_MSG_CHECKING([whether the datatype MPI_CHAR is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_CHAR;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_CHAR, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_CHAR, [MPI_CHAR])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_CHAR, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_SIGNED_CHAR],
		[MPI_SIGNED_CHAR if MPI_SIGNED_CHAR is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_SIGNED_CHAR], [defined if MPI_SIGNED_CHAR is available])
    AC_MSG_CHECKING([whether the datatype MPI_SIGNED_CHAR is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_SIGNED_CHAR;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_SIGNED_CHAR, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_SIGNED_CHAR, [MPI_SIGNED_CHAR])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_SIGNED_CHAR, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_UNSIGNED_CHAR],
		[MPI_UNSIGNED_CHAR if MPI_UNSIGNED_CHAR is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_UNSIGNED_CHAR], [defined if MPI_UNSIGNED_CHAR is available])
    AC_MSG_CHECKING([whether the datatype MPI_UNSIGNED_CHAR is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_UNSIGNED_CHAR;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_UNSIGNED_CHAR, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_UNSIGNED_CHAR, [MPI_UNSIGNED_CHAR])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_UNSIGNED_CHAR, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_BYTE],
		[MPI_BYTE if MPI_BYTE is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_BYTE], [defined if MPI_BYTE is available])
    AC_MSG_CHECKING([whether the datatype MPI_BYTE is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_BYTE;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_BYTE, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_BYTE, [MPI_BYTE])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_BYTE, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_WCHAR],
		[MPI_WCHAR if MPI_WCHAR is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_WCHAR], [defined if MPI_WCHAR is available])
    AC_MSG_CHECKING([whether the datatype MPI_WCHAR is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_WCHAR;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_WCHAR, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_WCHAR, [MPI_WCHAR])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_WCHAR, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_SHORT],
		[MPI_SHORT if MPI_SHORT is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_SHORT], [defined if MPI_SHORT is available])
    AC_MSG_CHECKING([whether the datatype MPI_SHORT is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_SHORT;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_SHORT, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_SHORT, [MPI_SHORT])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_SHORT, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_UNSIGNED_SHORT],
		[MPI_UNSIGNED_SHORT if MPI_UNSIGNED_SHORT is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_UNSIGNED_SHORT], [defined if MPI_UNSIGNED_SHORT is available])
    AC_MSG_CHECKING([whether the datatype MPI_UNSIGNED_SHORT is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_UNSIGNED_SHORT;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_UNSIGNED_SHORT, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_UNSIGNED_SHORT, [MPI_UNSIGNED_SHORT])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_UNSIGNED_SHORT, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_INT],
		[MPI_INT if MPI_INT is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_INT], [defined if MPI_INT is available])
    AC_MSG_CHECKING([whether the datatype MPI_INT is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_INT;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_INT, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_INT, [MPI_INT])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_INT, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_UNSIGNED],
		[MPI_UNSIGNED if MPI_UNSIGNED is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_UNSIGNED], [defined if MPI_UNSIGNED is available])
    AC_MSG_CHECKING([whether the datatype MPI_UNSIGNED is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_UNSIGNED;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_UNSIGNED, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_UNSIGNED, [MPI_UNSIGNED])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_UNSIGNED, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_LONG],
		[MPI_LONG if MPI_LONG is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_LONG], [defined if MPI_LONG is available])
    AC_MSG_CHECKING([whether the datatype MPI_LONG is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_LONG;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_LONG, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_LONG, [MPI_LONG])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_LONG, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_UNSIGNED_LONG],
		[MPI_UNSIGNED_LONG if MPI_UNSIGNED_LONG is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_UNSIGNED_LONG], [defined if MPI_UNSIGNED_LONG is available])
    AC_MSG_CHECKING([whether the datatype MPI_UNSIGNED_LONG is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_UNSIGNED_LONG;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_UNSIGNED_LONG, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_UNSIGNED_LONG, [MPI_UNSIGNED_LONG])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_UNSIGNED_LONG, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_FLOAT],
		[MPI_FLOAT if MPI_FLOAT is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_FLOAT], [defined if MPI_FLOAT is available])
    AC_MSG_CHECKING([whether the datatype MPI_FLOAT is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_FLOAT;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_FLOAT, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_FLOAT, [MPI_FLOAT])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_FLOAT, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_DOUBLE],
		[MPI_DOUBLE if MPI_DOUBLE is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_DOUBLE], [defined if MPI_DOUBLE is available])
    AC_MSG_CHECKING([whether the datatype MPI_DOUBLE is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_DOUBLE;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_DOUBLE, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_DOUBLE, [MPI_DOUBLE])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_DOUBLE, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_LONG_DOUBLE],
		[MPI_LONG_DOUBLE if MPI_LONG_DOUBLE is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_LONG_DOUBLE], [defined if MPI_LONG_DOUBLE is available])
    AC_MSG_CHECKING([whether the datatype MPI_LONG_DOUBLE is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_LONG_DOUBLE;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_LONG_DOUBLE, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_LONG_DOUBLE, [MPI_LONG_DOUBLE])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_LONG_DOUBLE, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_LONG_LONG_INT],
		[MPI_LONG_LONG_INT if MPI_LONG_LONG_INT is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_LONG_LONG_INT], [defined if MPI_LONG_LONG_INT is available])
    AC_MSG_CHECKING([whether the datatype MPI_LONG_LONG_INT is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_LONG_LONG_INT;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_LONG_LONG_INT, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_LONG_LONG_INT, [MPI_LONG_LONG_INT])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_LONG_LONG_INT, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_UNSIGNED_LONG_LONG],
		[MPI_UNSIGNED_LONG_LONG if MPI_UNSIGNED_LONG_LONG is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_UNSIGNED_LONG_LONG], [defined if MPI_UNSIGNED_LONG_LONG is available])
    AC_MSG_CHECKING([whether the datatype MPI_UNSIGNED_LONG_LONG is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_UNSIGNED_LONG_LONG;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_UNSIGNED_LONG_LONG, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_UNSIGNED_LONG_LONG, [MPI_UNSIGNED_LONG_LONG])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_UNSIGNED_LONG_LONG, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_LONG_LONG],
		[MPI_LONG_LONG if MPI_LONG_LONG is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_LONG_LONG], [defined if MPI_LONG_LONG is available])
    AC_MSG_CHECKING([whether the datatype MPI_LONG_LONG is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_LONG_LONG;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_LONG_LONG, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_LONG_LONG, [MPI_LONG_LONG])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_LONG_LONG, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_PACKED],
		[MPI_PACKED if MPI_PACKED is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_PACKED], [defined if MPI_PACKED is available])
    AC_MSG_CHECKING([whether the datatype MPI_PACKED is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_PACKED;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_PACKED, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_PACKED, [MPI_PACKED])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_PACKED, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_LB],
		[MPI_LB if MPI_LB is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_LB], [defined if MPI_LB is available])
    AC_MSG_CHECKING([whether the datatype MPI_LB is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_LB;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_LB, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_LB, [MPI_LB])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_LB, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_UB],
		[MPI_UB if MPI_UB is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_UB], [defined if MPI_UB is available])
    AC_MSG_CHECKING([whether the datatype MPI_UB is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_UB;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_UB, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_UB, [MPI_UB])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_UB, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_FLOAT_INT],
		[MPI_FLOAT_INT if MPI_FLOAT_INT is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_FLOAT_INT], [defined if MPI_FLOAT_INT is available])
    AC_MSG_CHECKING([whether the datatype MPI_FLOAT_INT is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_FLOAT_INT;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_FLOAT_INT, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_FLOAT_INT, [MPI_FLOAT_INT])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_FLOAT_INT, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_DOUBLE_INT],
		[MPI_DOUBLE_INT if MPI_DOUBLE_INT is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_DOUBLE_INT], [defined if MPI_DOUBLE_INT is available])
    AC_MSG_CHECKING([whether the datatype MPI_DOUBLE_INT is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_DOUBLE_INT;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_DOUBLE_INT, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_DOUBLE_INT, [MPI_DOUBLE_INT])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_DOUBLE_INT, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_LONG_INT],
		[MPI_LONG_INT if MPI_LONG_INT is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_LONG_INT], [defined if MPI_LONG_INT is available])
    AC_MSG_CHECKING([whether the datatype MPI_LONG_INT is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_LONG_INT;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_LONG_INT, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_LONG_INT, [MPI_LONG_INT])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_LONG_INT, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_SHORT_INT],
		[MPI_SHORT_INT if MPI_SHORT_INT is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_SHORT_INT], [defined if MPI_SHORT_INT is available])
    AC_MSG_CHECKING([whether the datatype MPI_SHORT_INT is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_SHORT_INT;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_SHORT_INT, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_SHORT_INT, [MPI_SHORT_INT])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_SHORT_INT, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_2INT],
		[MPI_2INT if MPI_2INT is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_2INT], [defined if MPI_2INT is available])
    AC_MSG_CHECKING([whether the datatype MPI_2INT is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_2INT;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_2INT, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_2INT, [MPI_2INT])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_2INT, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AH_TEMPLATE([DUMPI_DATATYPE_MPI_LONG_DOUBLE_INT],
		[MPI_LONG_DOUBLE_INT if MPI_LONG_DOUBLE_INT is defined, else MPI_DATATYPE_NULL])
    AH_TEMPLATE([DUMPI_HAVE_MPI_LONG_DOUBLE_INT], [defined if MPI_LONG_DOUBLE_INT is available])
    AC_MSG_CHECKING([whether the datatype MPI_LONG_DOUBLE_INT is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
	    int *argc;
	    char ***argv;
	    MPI_Datatype dt = MPI_LONG_DOUBLE_INT;
	    MPI_Init(argc, argv);
	    MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_LONG_DOUBLE_INT, [1])
        AC_DEFINE(DUMPI_DATATYPE_MPI_LONG_DOUBLE_INT, [MPI_LONG_DOUBLE_INT])
        AC_MSG_RESULT([yes])
      ], [
	AC_DEFINE(DUMPI_DATATYPE_MPI_LONG_DOUBLE_INT, [MPI_DATATYPE_NULL])
        AC_MSG_RESULT([no])
      ]
    )

    AC_LANG_POP
  ]
)
