/** \file
 * Python example app.
 * A very simple mpi skeleton application.
 */

#include "app.h"
#include <stdio.h>

namespace pyexample {

  // Public constructor
  // Creates a reference counted app object
  boost::shared_ptr<app>
  app::construct()
  {
    boost::shared_ptr<app> obj(new app());
    obj->set_self(obj);
    return obj;
  }

  // Get a copy
  // Launchers can use this to create app threads
  boost::shared_ptr<sstmac::mpiapp>
  app::clone()
  { return app::construct(); }

  void 
  app::run() {

    // initialize mpi
    if(mpi() == 0)
      throw sstmac::nullerror("app: mpiapi pointer is null.");
    sstmac::mpicomm world(mpi()->comm_world());
    sstmac::mpiid rank(world.rank());
    sstmac::mpiid size(world.size());
    mpi()->init();

    int ndata=1;
    sstmac::mpitag tag(TAG1);

    // rank - 1 is sending me a double
    sstmac::mpiid source;
    if( rank == sstmac::mpiid(0) ) source = sstmac::mpiid( size.id - 1 );
    else source = sstmac::mpiid( rank.id - 1 );
    sstmac::mpiapi::mpirequest_t rec_request;
    mpi()->irecv(ndata,sstmac::mpitype::mpi_double,
                 source,tag,world,rec_request);

    // send a double to the next higher rank
    sstmac::mpiid destination;
    if( rank.id == (size.id - 1) ) destination = sstmac::mpiid(0);
    else destination = sstmac::mpiid( rank.id + 1 ); 
    sstmac::mpiapi::mpirequest_t snd_request;
    mpi()->isend(ndata,sstmac::mpitype::mpi_double,
                 destination,tag,world,snd_request);

    // wait for send and receive to complete
    sstmac::mpiapi::const_mpistatus_t snd_status;
    mpi()->wait(snd_request,snd_status);
    sstmac::mpiapi::const_mpistatus_t rec_status;
    mpi()->wait(rec_request,rec_status);
    std::cout << "rank " << rank << " received message from rank " 
              << source << std::endl;

    // finalize
    mpi()->finalize();
  } 

} //end of namespace pyexample

