/** \file
 * Python example app.
 * A very simple mpi skeleton application.
 */

#include <sstmac/process/mpiapp.h>
#include <sstmac/process/mpiapi.h>

/// An example skeleton application using Python/SWIG.
namespace pyexample {

  /// MPI message tags
  enum tag_id {TAG1};

  /** app is an example mpi skeleton application. It inherits mpi
    * support [things like mpiapi()] from mpiapp. mpiapp is derived from
    * the thread class, so this will be a runnable task thread [it must
    * implement a run() method]. While this example will use a python 
    * driver script and swig interface to run the app, there is nothing
    * python specific about the app itself, it could just as well be 
    * run from a C++ executable (in the same manner as the hello-comm-world
    * example).
    */

  class app : public sstmac::mpiapp {

  protected:

    /** Protected constructor, public uses app::construct(). */
    app(){}; 

  public:
   
    /** Public constructor, creates a reference counted app object. */
    static boost::shared_ptr<app> construct();
    
    /** Word around for swig/python/user issues with multiple namespaces. */
    static boost::shared_ptr<sstmac::mpiapp> construct_as_mpiapp()
    { return construct(); }

    /** Get a copy, launchers can use this to create app threads. */
    boost::shared_ptr<sstmac::mpiapp> clone();

    /** Destructor. */
    ~app() throw() {};

    /** Overrides thread::run virtual function.
      * The task thread context is started by a call into this function. */
    void run();
  };

} // end of namespace pyexample

    
