/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/macrobridge/macrorecver.h>
//#include <sstmac/macrobridge/macrosender.h>
#include <sstmac/macrobridge/macroevent.h>
#include <sstmac/simulator/timestamp.h>
#include <sstmac/util/errors.h>
#include <iostream>

using namespace SST;

namespace sstmac {

  //
  // Private method to handle component events.
  //
  void macrorecver::handle_events(Event *evt) {
    macroevent *mevt = dynamic_cast<macroevent*>(evt);
    if(mevt == 0)
      throw valueerror("macrorecver::handle_events:  Invalid event type.");
    //std::cerr << "macrorecver::handle_events:  Event completion at time "
    //          << timestamp(now) << " (current time "
    //          << (port_->CurrentTime()) << ")\n";
    timestamp ts = timestamp::exact_ticks(getCurrentSimTime());
    //macrosender::set_now(ts);
    mevt->complete(ts);
    delete mevt;
  }

  //
  // Private method to add wiring between components.
  //
  void macrorecver::get_wired(const std::string &target) {
    // Templates gone wild.
    typedef Event::Handler<macrorecver> handler_t;
    handler_t *handler = new handler_t(this, &macrorecver::handle_events);
    port_ = configureLink(target, defaultTimeBase, handler);
    if(port_ == NULL) {
      throw illformederror("macrorecver::get_wired:  LinkAdd failed.");
    }
  }

  macrorecver::macrorecver(ComponentId_t id, Params_t &params) :
    Component(id)
  {
    registerTimeBase(sstmac::timestamp::tick_interval_string());

    //std::cerr << "macrorecver[" << this << "] constructing.\n"
    //          << "Parameters[" << params.size() << "]\n";
    std::string target = "";
    if(params.find("dest") != params.end())
      target = params["dest"];
    else
      throw valueerror("macrorecver: Parameter value \"dest\" is undefined.");
    if(target == "") {
      std::cerr << "macrorecver:  dest must not be a blank string.\n";
      throw valueerror("macrorecver::macrorecver:  Parameter value \"dest\" "
                       "is undefined.");
    }
    this->get_wired(target);

    //std::cerr << "macrorecver[" << this << "] constructed\n";
  }

  macrorecver::~macrorecver() {
    //std::cerr << "macrorecver[" << this << "] going out of scope\n";
  }

  //
  // Git goin'
  //
  int macrorecver::Setup() {
    std::cerr << "macrorecver[" << this << "]::Setup()\n";
    return 0;
  }

  //
  // Whoa!
  //
  int macrorecver::Finish() {
    std::cerr << "macrorecver[" << this << "]::Finish()\n";
    return 0;
  }

} // end of namespace sstmac
