/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */


// composite.h: Concrete class for product networks.

#ifndef SSTMAC_NETWORK_COMPOSITE_H
#define SSTMAC_NETWORK_COMPOSITE_H

#include <boost/shared_ptr.hpp>
#include <sstmac/network/product.h>
#include <sstmac/network/network.h>
#include <string>

namespace sstmac {

  class composite: public product {
  private:
    std::string name_;
    
    /** The composite network contructs a new network by taking the product
         of two networks. */
  protected:
    composite(const boost::shared_ptr<network>& n1,
              const boost::shared_ptr<network>& n2);
  public:
    static boost::shared_ptr<composite> construct(const boost::shared_ptr<network>& n1,
                                                  const boost::shared_ptr<network>& n2);
    virtual ~composite();
    std::string name() const;
  };
}

#endif
