/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */


// product.h: Interface for product networks.
//
// Author: Curtis Janssen <cljanss@ca.sandia.gov>

#ifndef SSTMAC_NETWORK_PRODUCT_H
#define SSTMAC_NETWORK_PRODUCT_H

#include <memory>
#include <sstmac/network/network.h>

namespace sstmac {

  /** The product network contructs a new network by taking the product of two
      networks. */
  class product: public network {
  private:
    boost::shared_ptr<network> n1_, n2_;
  protected:
    // called by derived class CTOR
    void assemble(const boost::shared_ptr<network>& n1, const boost::shared_ptr<network>& n2);

  protected:
    //product(TiXmlHandle) {}
    product() {}

  public:
    static boost::shared_ptr<product> construct();
    virtual ~product();

    int diameter() const;
    double a() const;
    int b() const;

    int shortest_path_length(const vertex &v1, const vertex &v2) const;
    void shortest_path(const vertex &v1, const vertex &v2, walk &) const;
    void next_hop(vertex_index loc, vertex_index dst,
                  std::vector<lwedge> &nxt) const;

    bool vertex_symmetric() const;
  };

}

#endif
