/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_ACTORDOWNMSG_H
#define SSTMAC_PROCESS_ACTORDOWNMSG_H

#include <sstmac/process/actormsg.h>
#include <sstmac/process/actorexit.h>

namespace sstmac {

  /**
   * A message in an actor-based application indicating that
   * a monitored process has exited.
   */
  class actordownmsg : public actormsg {
    /// What was the exit reason?
    boost::shared_ptr<const actorexit> why_;

    /// Create a new down message.
    actordownmsg(const boost::shared_ptr<const actorexit>& why) :
      why_(why)
    {}

  public:
    /// Construct a new message object.
    static const boost::shared_ptr<actordownmsg>&
    construct(const boost::shared_ptr<const actorexit>& why) {
      return boost::shared_ptr<actordownmsg>(why);
    }

    /// Goodbye.
    virtual ~actordownmsg() {
    }

    /// What's our reason?
    boost::shared_ptr<const actorexit> why() const {
      return why_;
    }
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_ACTORDOWNMSG_H
