/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_ACTORPATTERN_H
#define SSTMAC_PROCESS_ACTORPATTERN_H

#include <boost/shared_ptr.hpp>

namespace sstmac {

  // Forward declaration of important types.
  class actormsg;

  /**
   * Base type for pattern-matching on incoming messages.
   * Any test or recv call is pattern matched using a actorpattern.
   */
  class actorpattern {
    /// A nested pattern type that matches anything.
    class anypattern;
    /// A static object to match everything.
    static boost::shared_ptr<const actorpattern> any_;

  public:
    /// Goodbye.
    virtual ~actorpattern() {}

    /// Is this a match?
    /// \return true for a match.
    virtual bool matches(const boost::shared_ptr<const actormsg>& msg) const = 0;

    /// A special magical match that always returns true for a non-NULL message.
    static boost::shared_ptr<const actorpattern> any();
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_ACTORPATTERN_H
