/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_ACTORSERVER_CLIENT_LINKRESPONSE_H
#define SSTMAC_PROCESS_ACTORSERVER_CLIENT_LINKRESPONSE_H

#include <sstmac/process/actorserver-client-internalrequest.h>
#include <sstmac/process/actorserver-client-linkhandler.h>

namespace sstmac {

  /**
   * An internal message type used for actorserver::client.  This is a reply
   * from a node saying that our link request has been accepted.
   */
  class actorserver::client::linkresponse :
    public actorserver::client::internalrequest
  {
    /// The link request was sent with a unique identifier.
    int32_t linkid_;
    
    /// As always, the actual constructor is a private affair.
    linkresponse(int32_t linkid) : linkid_(linkid)
    {}

  public:
    /// Build a dynamically allocated, reference-counted object.
    static boost::shared_ptr<linkresponse> construct(int32_t linkid) {
      return boost::shared_ptr<linkresponse>(new linkresponse(linkid));
    }

    /// Our work here is done.
    virtual ~linkresponse() {
    }

    /// Handle the completion of this message.
    virtual void handle(const shared_ptr<actorserver::client>& parent,
                        const actorid &sender)
    {
      actorserver::client::linkreq_t::iterator it = parent->linkreq_.find(linkid_);
      if(linkid_ != parent->linkreq_.end()) {
        parent->second->response_received();
      }
    }
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_ACTORSERVER_CLIENT_LINKRESPONSE_H
