/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_APP_H
#define SSTMAC_PROCESS_APP_H

#include <boost/shared_ptr.hpp>
#include <sstmac/process/thread.h>

namespace sstmac {

  // Important types.
  class computeapi;

  /**
   * The app derived class adds to the thread base class by providing
   * facilities to allow applications to simulate computation.  
   * Messaging models are supported through derivation of this class,
   * as in the mpiapp class.
   */
  class app : virtual public thread {
    boost::shared_ptr<computeapi> computeapi_;

  protected:
    /// Access the compute API.
    boost::shared_ptr<computeapi>& compute_api()
      { return computeapi_; }
    
    /// Access the compute API in a const context.
    boost::shared_ptr<const computeapi> compute_api() const
      { return computeapi_; }
    
  public:
    /// Goodbye.
    virtual ~app() {}
    
    /// Initialize compute api.
    void init_computeapi(const boost::shared_ptr<computeapi>& api) 
    { computeapi_ = api; }

    /// Is compute api initialized?
    bool has_computeapi()
    { return computeapi_ != 0; }
  };
  
} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_APP_H
