/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPICOLLECTIVE_RECVER_H
#define SSTMAC_PROCESS_MPICOLLECTIVE_RECVER_H

#include <sstmac/process/mpicollective.h>
#include <sstmac/process/mpirecv.h>
#include <sstmac/process/mpitype.h>
#include <sstmac/process/mpitag.h>
#include <sstmac/process/mpiid.h>
#include <sstmac/process/mpimessage.h>
#include <sstmac/util/errors.h>
#include <sstmac/util/debug.h>

namespace sstmac {

  /**
   * A private mpirecv type to handle individual recv operations for
   * collective mpi operations.
   */
  class mpicollective::recver : public mpirecv {
    /// The parent collective operation.
    boost::shared_ptr<mpicollective> parent_;
    /// Number of elements sent.
    int count_;
    /// MPI type expected.
    mpitype type_;
    /// MPI tag associated with the operation.
    mpitag tag_;
    /// The peer from whom this message was received.
    mpiid source_;

  protected:
    /// Hi.
    recver(const boost::shared_ptr<mpicollective>& parent,
           int count, const mpitype &type,
           const mpitag &tag, const mpiid &source) :
      parent_(parent), count_(count),
      type_(type), tag_(tag), source_(source)
    {}

  public:
    /// Hello.
    static boost::shared_ptr<recver>
    construct(const boost::shared_ptr<mpicollective>& parent,
           int count, const mpitype &type,
           const mpitag &tag, const mpiid &source)
    {
      return boost::shared_ptr<recver>
        (new recver(parent, count, type, tag, source));
    }

    /// Goodbye
    virtual ~recver() throw() {
    }

    /// Event handler callback.
    void handle(const timestamp &now) {
      if(debugger::active())
        debug() << "mpicollective::recver[" << this
                << "]::handle(" << now << ")\n";
      parent_->recv_complete(now, count_, this->get_message()->build_status());
    }
  };
  
} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_MPICOLLECTIVE_RECVER_H
