/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPICOLLECTIVE_SENDER_H
#define SSTMAC_PROCESS_MPICOLLECTIVE_SENDER_H

#include <sstmac/process/mpicollective.h>
#include <sstmac/process/mpisend.h>
#include <sstmac/process/mpitype.h>
#include <sstmac/process/mpitag.h>
#include <sstmac/process/mpiid.h>
#include <sstmac/util/errors.h>
#include <sstmac/util/debug.h>

namespace sstmac {

  /**
   * A private mpisend type to handle individual send operations for
   * collective mpi operations.
   */
  class mpicollective::sender : public mpisend {
    /// The parent collective operation.
    boost::shared_ptr<mpicollective> parent_;
    /// The MPI status object associated with the collective operation.
    boost::shared_ptr<mpistatus> status_;
    /// Send length
    int count_;
    /// Send type.
    mpitype type_;
    /// Send tag.
    mpitag tag_;
    /// Destination
    mpiid dest_;

  protected:
    /// Hi.
    sender(const boost::shared_ptr<mpicollective>& parent,
           const boost::shared_ptr<mpistatus>& status,
           int count, const mpitype &type,
           const mpitag &tag, const mpiid &dest) :
      parent_(parent), status_(status), count_(count),
      type_(type), tag_(tag), dest_(dest)
    {}

  public:
    /// Hello.
    static boost::shared_ptr<sender>
    construct(const boost::shared_ptr<mpicollective>& parent,
              const boost::shared_ptr<mpistatus>& status,
              int count, const mpitype &type,
              const mpitag &tag, const mpiid &dest)
    {
      return boost::shared_ptr<sender>
        (new sender(parent, status, count, type, tag, dest));
    }

    /// Goodbye
    virtual ~sender() throw() {
    }

    /// Access the mpistatus.
    boost::shared_ptr<mpistatus> get_mpistatus() {
      return status_;
    }

    /// Event handler callback.
    void handle(const timestamp &now) {
      if(debugger::active())
        debug() << "mpicollective::sender[" << this
                << "]::handle(" << now << ")\n";
      parent_->send_complete(now, count_, type_, tag_, dest_);
    }
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_MPICOLLECTIVE_SENDER_H
