/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpicomm.h>
#include <sstmac/process/mpicommdata.h>
#include <sstmac/util/debug.h>
#include <sstmac/util/errors.h>

namespace sstmac {

  using boost::shared_ptr;
  
  //
  // NULL communicator.
  //
  const mpicomm mpicomm::comm_null;

  //
  // Build a 'self' communicator.
  //
  mpicomm mpicomm::construct_self(const mpicomm &other) {
    mpiid id = other.rank();
    std::vector<taskid> tid(1);
    tid[0] = other.data_->peers().at(id.id);
    std::vector<nodeid> nid(1);
    nid[0] = other.data_->nodes().at(id.id);
    return mpicomm(mpiid(0), mpicommdata::construct(mpicommid(-1), tid, nid));
  }


  //
  // Default constructor.
  //
//   mpicomm::mpicomm() :
//     rank_(-1), data_()
//   {}

  //
  // Hello.
  //
//   mpicomm::mpicomm(const mpiid &rank,
//                    boost::shared_ptr<const mpicommdata> data) :
//     rank_(rank), data_(data)
//   {}

  //
  // Goodbye.
  //
//   mpicomm::~mpicomm() {
//   }

  //
  // The rank of this process in the communicator.
  //
  //const mpiid& mpicomm::rank() const {
  //  return rank_;
  //}

  //
  // The size of the communicator.
  //
  //mpiid mpicomm::size() const {
  //  if(data_==0) throw nullerror("mpicomm::size: data pointer is null.");
  //  return mpiid(data_->peers().size());
  //}

  //
  // The communicator id.
  //
  //const mpicommid& mpicomm::id() const {
  //  if(data_==0) throw nullerror("mpicomm::id: data pointer is null.");
  //  return data_->id();
  //}

  //
  // Get this task id.
  //
  //taskid mpicomm::my_task() const {
  //  return this->peers().at(rank_.id);
  //}

  //
  // Get a specific peer.
  //
  //taskid mpicomm::peer_task(const mpiid &id) const {
  //  if(debugger::active()) debug() << "mpicomm::peer_task(" << id << ")\n";
  //  if((id.id < 0) || (id >= this->size()))
  //    throw valueerror("mpicomm::peer:  Index out of range.");
  //  return this->peers().at(id.id);
  //}

  //
  // My peers.
  //
  //const std::vector<taskid>& mpicomm::peers() const {
  //  if(data_==0) throw nullerror("mpicomm::peers: data pointer is null.");
  //  return data_->peers();
  //}

  //
  // Fairly self-explanatory.
  //
  std::ostream& operator<<(std::ostream &os, const mpicomm &comm) {
    os << "mpicomm(rank=" << comm.rank_ << ", data=" << comm.data_ << ")";
    return os;
  }

} // end of namespace sstmac
