/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpicommid.h>
#include <sstmac/util/errors.h>
#include <iostream>

namespace sstmac {

  //
  // Hello.
  //
  mpicommid::mpicommid(int idit) :
    id(idit)
  {}

  //
  // Test two ids for (in)equality.
  //
  bool operator==(const mpicommid &a, const mpicommid &b) {
    return a.id == b.id;
  }
  bool operator!=(const mpicommid &a, const mpicommid &b) {
    return !(a == b);
  }

  //
  // Rank-compare operations.
  //
  bool operator<(const mpicommid &a, const mpicommid &b) { return (a.id < b.id); }
  bool operator>(const mpicommid &a, const mpicommid &b) { return (a.id > b.id); }
  bool operator<=(const mpicommid &a, const mpicommid &b){ return (a.id <= b.id); }
  bool operator>=(const mpicommid &a, const mpicommid &b){ return (a.id >= b.id); }

  //
  // Fairly self-explanatory.
  //
  std::ostream& operator<<(std::ostream &os, const mpicommid &id) {
    os << "mpicommid(id=" << id.id << ")";
    return os;
  }

} // end of namespace sstmac
