/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpicoreallgatherv.h>
#include <sstmac/process/mpiringallgatherv.h>
#include <sstmac/util/errors.h>
#include <sstmac/util/debug.h>

namespace sstmac {

  // Pointer types.
  using boost::shared_ptr;

  //
  // Hi.
  //
  mpicoreallgatherv::mpicoreallgatherv() {
  }

  //
  // Hello.
  //
  shared_ptr<mpicoreallgatherv> mpicoreallgatherv::construct() {
    return shared_ptr<mpicoreallgatherv>(new mpicoreallgatherv);
  }

  //
  // Goodbye.
  //
  mpicoreallgatherv::~mpicoreallgatherv() throw() {
  }

  //
  // Build a kernel for an allgatherv operation
  //
  shared_ptr<mpikernel>
  mpicoreallgatherv::build(const shared_ptr<key>& thekey,
                           const shared_ptr<mpiqueue>& queue,
                           int sendcount, const mpitype &sendtype,
                           const std::vector<int> &recvcount,
                           const mpitype &recvtype,
                           const mpitag &tag, const mpicomm &comm,
                           const shared_ptr<const payload>& content,
                           const shared_ptr<mpimagic>& magic) const
  {
    return mpiringallgatherv::construct(thekey, queue, sendcount, sendtype,
                                        recvcount, recvtype, tag, comm,
                                        content, magic);
  }

} // end of namespace sstmac
