/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpicorebarrier.h>
#include <sstmac/process/mpibarrierengine.h>
#include <sstmac/util/hfmiterator.h>
#include <sstmac/util/errors.h>
#include <sstmac/util/debug.h>

namespace sstmac {

  // Pointer types.
  using boost::shared_ptr;

  //
  // Hello.
  //
  mpicorebarrier::mpicorebarrier() {
  }

  //
  // Hi.
  //
  shared_ptr<mpicorebarrier> mpicorebarrier::construct() {
    return shared_ptr<mpicorebarrier>(new mpicorebarrier);
  }

  //
  // Goodbye.
  //
  mpicorebarrier::~mpicorebarrier() throw() {
  }

  //
  // Build a kernel to synchronize nodes.
  //
  boost::shared_ptr<mpikernel>
  mpicorebarrier::build(const boost::shared_ptr<key>& thekey,
                        const boost::shared_ptr<mpiqueue>& queue,
                        const mpitag &tag, const mpicomm &comm,
                        const boost::shared_ptr<mpimagic>& /*magic is ignored*/) const
  {
    // Here we might want to pick different iterators depending on message size.
    shared_ptr<topologyiterator> iter =
      hfmiterator::construct(comm.rank().id, comm.size().id);
    return mpibarrierengine::construct(iter, thekey, queue, tag, comm);
  }

} // end of namespace sstmac
