/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpicoregather.h>
#include <sstmac/process/mpibtreegather.h>
#include <sstmac/util/errors.h>
#include <sstmac/util/debug.h>

namespace sstmac {

  //
  // Hi.
  //
  mpicoregather::mpicoregather() {
  }

  //
  // Hello.
  //
  boost::shared_ptr<mpicoregather> mpicoregather::construct() {
    return boost::shared_ptr<mpicoregather>(new mpicoregather);
  }

  //
  // Perform a fused reduce-gather operation
  //
  boost::shared_ptr<mpikernel>
  mpicoregather::build(const boost::shared_ptr<key>& thekey,
                       const boost::shared_ptr<mpiqueue>& queue,
                       int sendcount, const mpitype &sendtype,
                       int recvcount, const mpitype &recvtype,
                       const mpiid &root, const mpitag &tag,
                       const mpicomm &comm,
                       const boost::shared_ptr<const payload>& content,
                       const boost::shared_ptr<mpimagic>& magic) const
  {
    return mpibtreegather::construct(thekey, queue, sendcount, sendtype,
                                     recvcount, recvtype, root, tag, comm,
                                     content, magic);
  }

} // end of namespace sstmac
