/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpicorescatter.h>
#include <sstmac/process/mpibtreescatter.h>
#include <sstmac/util/errors.h>
#include <sstmac/util/debug.h>

namespace sstmac {

  using boost::shared_ptr;

  //
  // Hi.
  //
  mpicorescatter::mpicorescatter() {
  }

  //
  // Hello.
  //
  shared_ptr<mpicorescatter> mpicorescatter::construct() {
    return shared_ptr<mpicorescatter>(new mpicorescatter);
  }

  //
  // Perform a fused reduce-scatter operation
  //
  shared_ptr<mpikernel>
  mpicorescatter::build(const shared_ptr<key>& thekey,
                        const shared_ptr<mpiqueue>& queue,
                        int sendcount, const mpitype &sendtype,
                        int recvcount, const mpitype &recvtype,
                        const mpiid &root, const mpitag &tag,
                        const mpicomm &comm,
                        const shared_ptr<const mpicollpayload>& cc, 
                        const shared_ptr<mpimagic>& /*magic ignored*/) const
  {
    return mpibtreescatter::construct(thekey, queue, sendcount, sendtype,
                                      recvcount, recvtype, root, tag, comm, cc);
  }

} // end of namespace sstmac
