/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPIHANDLER_H_
#define SSTMAC_PROCESS_MPIHANDLER_H_

#include <sstmac/process/mpiotf.h>

namespace sstmac
{

  class adapter
  {
  public:
    virtual ~adapter() {}
    virtual void makecall( double time) {};
  };

  // Adapter for the function MPI_Send.
  class cMPI_Send : public adapter
  {
    mpiotf *parent_;
  public:
  cMPI_Send(mpiotf *parent) : parent_(parent) {}
    virtual ~cMPI_Send() {}
    void makecall( double time)
    { parent_->otfMPI_Send( time); }
  };

  // Adapter for the function MPI_Recv.
  class cMPI_Recv : public adapter
  {
    mpiotf *parent_;
  public:
  cMPI_Recv(mpiotf *parent) : parent_(parent) {}
    virtual ~cMPI_Recv() {}
    void makecall(double time) { parent_->otfMPI_Recv(time); }
  };

  // Adapter for the function MPI_Wait.
  class cMPI_Wait : public adapter
  {
    mpiotf *parent_;
  public:
  cMPI_Wait(mpiotf *parent) : parent_(parent) {}
    virtual ~cMPI_Wait() {}
    void makecall(double time)
    { parent_->otfMPI_Wait(time); }
  };

  // Adapter for the function MPI_Waitall.
  class cMPI_Waitall : public adapter
  {
    mpiotf *parent_;
  public:
  cMPI_Waitall(mpiotf *parent) : parent_(parent) {}
    virtual ~cMPI_Waitall() {}
    void makecall(double time) { parent_->otfMPI_Waitall(time); }
  };

  // Adapter for the function MPI_Isend.
  class cMPI_Isend : public adapter
  {
    mpiotf *parent_;
  public:
  cMPI_Isend(mpiotf *parent) : parent_(parent) {}
    virtual ~cMPI_Isend() {}
    void makecall(double time) { parent_->otfMPI_Isend(time); }
  };

  // Adapter for the function MPI_Irecv.
  class mpiotf::cMPI_Irecv : public mpiotf::adapter
    {
      mpiotf *parent_;
    public:
    cMPI_Irecv(mpiotf *parent) : parent_(parent) {}
      virtual ~cMPI_Irecv() {}
      void makecall(double time)
      { parent_->otfMPI_Irecv(time); }
    };

  // Adapter for the function MPI_Rsend
  class mpiotf::cMPI_Rsend : public mpiotf::adapter
    {
      mpiotf *parent_;
    public:
    cMPI_Rsend(mpiotf *parent) : parent_(parent) {}
      virtual ~cMPI_Rsend() {}
      void makecall(double time) { parent_->otfMPI_Rsend(time); }
    };

  // Adapter for the function MPI_Finalize.
  class mpiotf::cMPI_Finalize : public mpiotf::adapter
    {
      mpiotf *parent_;
    public:
    cMPI_Finalize(mpiotf *parent) : parent_(parent) {}
      virtual ~cMPI_Finalize() {}
      void makecall(double time) { parent_->otfMPI_Finalize(time); }
    };

  // Adapter for the function MPI_Allreduce.
  class mpiotf::cMPI_Allreduce : public mpiotf::adapter
    {
      mpiotf *parent_;
    public:
    cMPI_Allreduce(mpiotf *parent) : parent_(parent) {}
      virtual ~cMPI_Allreduce() {}
      void makecall(double time) { parent_->otfMPI_Allreduce( time); }
    };

  // Adapter for the function MPI_Reduce_scatter.
  class mpiotf::cMPI_Reduce_scatter : public mpiotf::adapter
    {
      mpiotf *parent_;
    public:
    cMPI_Reduce_scatter(mpiotf *parent) : parent_(parent) {}
      virtual ~cMPI_Reduce_scatter() {}
      void makecall(double time) { parent_->otfMPI_Reduce_scatter(time); }
    };

  // Adapter for the function MPI_Init.
  class mpiotf::cMPI_Init : public mpiotf::adapter
    {
      mpiotf *parent_;
    public:
    cMPI_Init(mpiotf *parent) : parent_(parent) {}
      virtual ~cMPI_Init() {}
      void makecall(double time) { parent_->otfMPI_Init(time); }
    };

  // Adapter for the function MPI_Comm_size.
  class mpiotf::cMPI_Comm_size : public mpiotf::adapter
    {
      mpiotf *parent_;
    public:
    cMPI_Comm_size(mpiotf *parent) : parent_(parent) {}
      virtual ~cMPI_Comm_size() {}
      void makecall(double time) { parent_->otfMPI_Comm_size(time); }
    };
  // Adapter for the function MPI_Comm_rank.
  class mpiotf::cMPI_Comm_rank : public mpiotf::adapter
    {
      mpiotf *parent_;
    public:
    cMPI_Comm_rank(mpiotf *parent) : parent_(parent) {}
      virtual ~cMPI_Comm_rank() {}
      void makecall(double time) { parent_->otfMPI_Comm_rank(time); }
    };

  // Adapter for the function MPI_Allgather.
  class mpiotf::cMPI_Allgather : public mpiotf::adapter
    {
      mpiotf *parent_;
    public:
    cMPI_Allgather(mpiotf *parent) : parent_(parent) {}
      virtual ~cMPI_Allgather() {}
      void makecall(double time) { parent_->otfMPI_Allgather(time); }
    };

  // Adapter for the function MPI_Barrier.
  class mpiotf::cMPI_Barrier : public mpiotf::adapter
    {
      mpiotf *parent_;
    public:
    cMPI_Barrier(mpiotf *parent) : parent_(parent) {}
      virtual ~cMPI_Barrier() {}
      void makecall(double time) { parent_->otfMPI_Barrier(time); }
    };

  // Adapter for the function MPI_Bcast.
  class mpiotf::cMPI_Bcast : public mpiotf::adapter
    {
      mpiotf *parent_;
    public:
    cMPI_Bcast(mpiotf *parent) : parent_(parent) {}
      virtual ~cMPI_Bcast() {}
      void makecall(double time) { parent_->otfMPI_Bcast(time); }
    };

}
#endif

