/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPIOP_H
#define SSTMAC_PROCESS_MPIOP_H

#include <sstmac/simulator/timestamp.h>
#include <string>

namespace sstmac {

  /*
   * Operations for simulating data reduction.
   */
  struct mpiop {
    std::string label;
    timestamp cost_per_entry;

    explicit mpiop(const std::string &labelit = "", const timestamp &costit = timestamp(0));

    // Common reduction operations.
    static const mpiop max;
    static const mpiop min;
    static const mpiop sum;
    static const mpiop prod;
    static const mpiop land;
    static const mpiop band;
    static const mpiop lor;
    static const mpiop bor;
    static const mpiop lxor;
    static const mpiop bxor;
    static const mpiop minloc;
    static const mpiop maxloc;
    static const mpiop replace;
    static const mpiop op_null;
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_MPIOP_H
