/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPIQUEUE_SENDREQUEST_H
#define SSTMAC_PROCESS_MPIQUEUE_SENDREQUEST_H

#include <sstmac/process/mpiqueue.h>
#include <sstmac/process/messageid.h>
#include <sstmac/util/nocopy.h>

namespace sstmac {

  // forward declaration of important types.
  class payload;

  /**
   * A nested type to handle individual send operations.
   */
  class mpiqueue::sendrequest : virtual public nocopy {
    /// Me.
    boost::weak_ptr<sendrequest> self_;
    /// The queue.
    boost::weak_ptr<mpiqueue> queue_;
    /// The size at which we switch between ready-send and handshake.
    /// If ready-send is specified at construction, handshaking is not used.
    int64_t handsize_;
    int64_t envelope_;
    int64_t mintrans_;
    /// The transmission paramters.
    int count_;
    mpitype type_;
    mpiid dest_;
    mpitag tag_;
    mpicomm comm_;
    bool rsend_;
    bool ssend_;
    mpimessage::category cat_;
    int seqno_;
    messageid id_;
    /// The object that gets notified when we're done.
    boost::shared_ptr<mpisend> completion_;
    boost::shared_ptr<const payload> content_;
    /// Status flag to test whether we're waiting for an ack.
    bool pending_ack_;
    bool data_sent_;

  protected:
    /// Hello.
    sendrequest(const boost::shared_ptr<mpiqueue>& queue, int64_t handsize,
                int64_t envelope, int64_t mintrans, int count,
                const mpitype &type, const mpiid &dest, const mpitag& tag,
                const mpicomm &comm, bool rsend, bool ssend, 
                mpimessage::category message_type,
                int sequence_number, const messageid &id,
                const boost::shared_ptr<mpisend>& completion,
                const boost::shared_ptr<const payload>& content);

    /// Who am I?
    void set_self(const boost::shared_ptr<sendrequest>& self);

    /// This is who I am.
    boost::shared_ptr<sendrequest> get_self();

  public:
    /// Hi there.
    static boost::shared_ptr<sendrequest>
    construct(const boost::shared_ptr<mpiqueue>& queue, int64_t handsize,
              int64_t envelope, int64_t mintrans, int count,
              const mpitype &type, const mpiid &dest, const mpitag& tag,
              const mpicomm &comm, bool rsend, bool ssend, 
              mpimessage::category message_type,
              int sequence_number, const messageid &id,
              const boost::shared_ptr<mpisend>& completion,
              const boost::shared_ptr<const payload>& content);

    /// Goodbye.
    virtual ~sendrequest() throw();

    /// Go.
    void start(const boost::shared_ptr<mpiserver>& lib);

    /// We just received an ack.
    void ack(const timestamp &current_time,
             const boost::shared_ptr<mpimessage>& response,
             const boost::shared_ptr<mpiserver>& lib);

    /// Eventhandler completion.
    //void complete(const timestamp &current_time);
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_MPIQUEUE_SENDREQUEST_H
