/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPIRECV_H
#define SSTMAC_PROCESS_MPIRECV_H

#include <sstmac/simulator/eventhandler.h>

namespace sstmac {

  // Important types.
  class mpimessage;

  /**
   * A base for event handlers that manages recv requests.
   */
  class mpirecv : public eventhandler {
    /// The message (filled in when this request is complete).
    boost::shared_ptr<mpimessage> message_;

  protected:
    /// Hello.
    mpirecv();

  public:
    /// Goodbye.
    virtual ~mpirecv() throw();

    /// Assign a message with this object.
    void set_message(const boost::shared_ptr<mpimessage>& msg);

    /// Access the message associated with this object.
    boost::shared_ptr<mpimessage> get_message();

    /// You still need to fill in the event handler callback.
    using eventhandler::handle;
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_MPIRECV_H
