/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPISEND_H
#define SSTMAC_PROCESS_MPISEND_H

#include <sstmac/simulator/eventhandler.h>

namespace sstmac {

  // Important types.
  class mpistatus;

  /**
   * A base for event handlers that manages send requests.
   */
  class mpisend : public eventhandler {
    /// The status associated with this send operation.
    boost::shared_ptr<mpistatus> status_;

  protected:
    /// Hello.
    mpisend();

  public:
    /// Goodbye.
    virtual ~mpisend() throw();

    /// Get the status associated with this send request.
    boost::shared_ptr<mpistatus> get_mpistatus();

    /// You still need to fill in the event handler callback.
    using eventhandler::handle;
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_MPISEND_H
