/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpiserver-localtransfer.h>
#include <sstmac/process/mpiqueue.h>
#include <sstmac/util/errors.h>

namespace sstmac {

  //
  // Hi.
  //
  mpiserver::localtransfer::
  localtransfer(const boost::shared_ptr<mpiqueue>& queue,
                const boost::shared_ptr<mpimessage>& payload,
                const boost::shared_ptr<eventhandler>& senddone) :
    queue_(queue), payload_(payload), senddone_(senddone)
  {
    if(queue_ == 0) {
      throw nullerror("mpiserver::localtransfer:  null queue pointer.");
    }
  }

  //
  // Hello.
  //
  boost::shared_ptr<mpiserver::localtransfer>
  mpiserver::localtransfer::construct(const boost::shared_ptr<mpiqueue>& queue,
                                      const boost::shared_ptr<mpimessage>& payload,
                                      const boost::shared_ptr<eventhandler>& senddone)
  {
    return boost::shared_ptr<localtransfer>
      (new localtransfer(queue, payload, senddone));
  }

  //
  // Goodbye.
  //
  mpiserver::localtransfer::~localtransfer() throw() {
  }

  //
  // Event.
  //
  void mpiserver::localtransfer::handle(const timestamp &now) {
    senddone_->handle(now);
    queue_->incoming_message(now, payload_);
  }

} // end of namespace sstmac.
