/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpitag.h>
#include <sstmac/util/errors.h>
#include <iostream>

namespace sstmac {

  //
  // The static "any tag" tag.
  //
  const mpitag mpitag::any_tag(-1);

  //
  // Tag MPI requests (this is just an int in the MPI spec).
  //
  mpitag::mpitag(int idit) :
    id(idit)
  {}

  //
  // Test whether two mpi tags match.
  //
  bool mpitag::matches(const mpitag &other) const {
    return ((this->id == any_tag.id) || (this->id == other.id));
  }

  //
  // (In)equality comparison (stricter than matches).
  //
  bool operator==(const mpitag &a, const mpitag &b) {
    return (a.id == b.id);
  }
  bool operator!=(const mpitag &a, const mpitag &b) {
    return !(a == b);
  }
  bool operator<(const mpitag &a, const mpitag &b) {
    return (a.id < b.id);
  }

  //
  // Fairly self-explanatory.
  //
  std::ostream& operator<<(std::ostream &os, const mpitag &tag) {
    os << "mpitag(id=" << tag.id << ")";
    return os;
  }

} // end of namespace sstmac
