/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/simpleapp.h>
#include <sstmac/process/key.h>
#include <sstmac/process/processrequest.h>
#include <sstmac/processor/eventdata.h>
#include <sstmac/processor/computeapi.h>

namespace sstmac {

  // Public constructor
  boost::shared_ptr<app> simpleapp::construct()
  {
    boost::shared_ptr<simpleapp> obj( new simpleapp() );
    obj->set_self(obj);
    return obj;
  }

  /// Goodbye.
  simpleapp::~simpleapp() throw() {} 

  /// Go.
  void simpleapp::run() 
  {

    // 
    // Do a couple sequential computes
    //

    // create simple processor input
    boost::shared_ptr<eventdata> instructions = eventdata::construct();
  
    // schedule first kernel
    instructions->set_event("PAPI_FP_INS",1000);
    instructions->set_event("PAPI_LST_INS",5);
    compute_api()->compute(instructions);

    // schedule second kernel
    instructions->set_event("PAPI_FP_INS",2000);
    instructions->set_event("PAPI_LST_INS",100);
    compute_api()->compute(instructions);

    //
    // Do a couple simultaneous computes (no wait)
    // 

    boost::shared_ptr<eventdata> instructions4 =
      eventdata::construct();
    instructions4->set_event("PAPI_FP_INS",2000);
    instructions4->set_event("PAPI_LST_INS",100);
 
    // process third kernel
    instructions->set_event("PAPI_FP_INS",1000);
    instructions->set_event("PAPI_LST_INS",5);
    compute_api()->compute(instructions);

    // process fourth kernel
    instructions->set_event("PAPI_FP_INS",2000);
    instructions->set_event("PAPI_LST_INS",100);
    compute_api()->compute(instructions);

  }

} // end of namespace sstmac
