/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_CIRCUIT_TRANSFER_FINISH_H
#define SSTMAC_SIMULATOR_CIRCUIT_TRANSFER_FINISH_H

#include <sstmac/simulator/circuit-transfer.h>
#include <sstmac/simulator/circuit-edge.h>

namespace sstmac {

  /**
   * Release the edges in a circuit simulator.
   */
  class circuit::transfer::finish : public eventhandler {
    /// The parent circuit.
    boost::shared_ptr<circuit> parent_;
    /// The edges we will be releasing.
    typedef std::vector<int> edges_t;
    edges_t edges_;

  protected:
    /// Hi.
    finish(const boost::shared_ptr<circuit> &parent, const std::vector<int> &edges) :
      parent_(parent), edges_(edges)
    {}

  public:
    /// Hi.
    static boost::shared_ptr<finish>
    construct(const boost::shared_ptr<circuit> &parent, const std::vector<int> &edges)
    {
      return boost::shared_ptr<finish>(new finish(parent, edges));
    }

    /// Bye.
    virtual ~finish() throw() {}

    /// All done.
    void handle(const timestamp &) {
      if (debugger::active()) debug() << "circuit::transfer::finish[" << this << "]::handle.  "
              << "Releasing " << edges_.size() << " edges.\n";
      for(edges_t::const_iterator it=edges_.begin(); it!=edges_.end(); ++it) {
        if (debugger::active()) debug() <<"circuit::transfer::finish["<<this<<"]::handle:"
                << "  releasing edge " << &(parent_->edge_.at(*it)) << "\n";
        parent_->edge_.at(*it).release();
      }
    }
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SIMULATOR_CIRCUIT_TRANSFER_FINISH_H
