/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_CONSTANTBANDWIDTH_H
#define SSTMAC_SIMULATOR_CONSTANTBANDWIDTH_H

#include <sstmac/simulator/bandwidthmodel.h>
#include <boost/shared_ptr.hpp>

namespace sstmac {

  /**
   * The simplest bandwidth model.  All transfers are assumed to have
   * the same constant bandwidth.
   */
  class constantbandwidth : public bandwidthmodel {
    /// The bandwidth we dole out.
    bandwidth bandwidth_;
    
    /// And off we go.
    constantbandwidth(bandwidth bw) :
      bandwidth_(bw)
    {}

  public:
    /// May we offer you some constant bandwidth?
    /// \param bw  bandwidth in bytes/ps
    static boost::shared_ptr<constantbandwidth> construct(bandwidth bw) {
      return boost::shared_ptr<constantbandwidth>(new constantbandwidth(bw));
    }

    /// No more Mr. Nice Constant Bandwidth.
    virtual ~constantbandwidth() {}

    /// What sort of bandwidth are we talking about?
    /// \return bandwidth in bytes/ps.
    virtual bandwidth bw(data) const {
      return bandwidth_;
    }
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_CONSTANTBANDWIDTH_H
