/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_CONSTANTLATENCY_H
#define SSTMAC_SIMULATOR_CONSTANTLATENCY_H

#include <sstmac/simulator/latencymodel.h>
#include <boost/shared_ptr.hpp>

namespace sstmac {

  /**
   * The simplest latency model.  All transfers are assumed to have
   * a constant latency.
   */
  class constantlatency : public latencymodel {
    /// The latency we dole out.
    timestamp latency_;

    /// And off we go.
    constantlatency(const timestamp &lat) :
      latency_(lat)
    {}

  public:
    /// May we offer you some constant latency?
    static boost::shared_ptr<constantlatency> construct(const timestamp &lat) {
      return boost::shared_ptr<constantlatency>(new constantlatency(lat));
    }

    /// No more Mr. Nice Constant Latency.
    virtual ~constantlatency() {}

    /// What sort of latency are we talking about?
    virtual timestamp latency(data) const {
      return latency_;
    }
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_CONSTANTLATENCY_H
