/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_INTERCONNECT_H
#define SSTMAC_SIMULATOR_INTERCONNECT_H

#include <sstmac/simulator/timestamp.h>
#include <sstmac/simulator/nodeid.h>
#include <sstmac/util/nocopy.h>
#include <boost/shared_ptr.hpp>
#include <boost/weak_ptr.hpp>

namespace sstmac {

  // Important types.
  class eventmanager;
  class eventhandler;

  /**
   * Base class for network simulators.
   */
  class interconnect : virtual public nocopy {
  public:
    virtual ~interconnect() {}

    /// The maximum number of nodes on this network.
    /// Return a negative (really <= 0) value to indicate an infinite capacity.
    virtual int slots() const = 0;

    /// The main event manager for the simulation.
    virtual void set_eventmanager(const boost::shared_ptr<eventmanager> &mgr) = 0;

    /// Send the given message at the given time.
    /// Notify the sendhandler when the send is complete.
    /// Notify the recvhandler when the recv is complete.
    virtual void send(const timestamp &start_time,
                      int64_t bytes,
                      nodeid sender,
                      nodeid recver,
                      const boost::shared_ptr<eventhandler> &sendhandler,
                      const boost::shared_ptr<eventhandler> &recvhandler) = 0;
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SIMULATOR_INTERCONNECT_H
