/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_KERNELCOMPLETION_H
#define SSTMAC_SIMULATOR_KERNELCOMPLETION_H

#include <sstmac/simulator/eventhandler.h>
#include <sstmac/simulator/kernel.h>

namespace sstmac {

  /**
   * Basic event handler to signal completion of a kernel.
   */
  class kernelcompletion : public eventhandler {
    boost::shared_ptr<kernel> kernel_;
  protected:
    kernelcompletion(const boost::shared_ptr<kernel> &ker) : kernel_(ker) {}
  public:
    static boost::shared_ptr<kernelcompletion>
    construct(const boost::shared_ptr<kernel> &ker) {
      return boost::shared_ptr<kernelcompletion>(new kernelcompletion(ker));
    }
    virtual ~kernelcompletion() throw() {
    }
    void handle(const timestamp &current_time) {
      kernel_->complete(current_time);
    }
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_SIMULATOR_KERNELCOMPLETION_H
