/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/simulator/kernelhandler.h>
#include <sstmac/simulator/kernel.h>
#include <sstmac/simulator/nodeinterface.h>
#include <sstmac/util/errors.h>

namespace sstmac {

  using boost::shared_ptr;

  //
  // Hello
  //
  kernelhandler::kernelhandler(const shared_ptr<kernel> &the_kernel,
                               const shared_ptr<nodeinterface> &the_delegate_) :
    kernel_(the_kernel), delegate_(the_delegate_)
  {}

  //
  // Who am I?
  //
  void kernelhandler::set_self(const shared_ptr<kernelhandler> &self) {
    if(self.get() != this) {
      throw valueerror("kernelhandler::set_self:  'self' pointer does not "
                       "point to 'this' object.");
    }
    self_ = self;
  }

  //
  // This is who I am.
  //
  shared_ptr<kernelhandler> kernelhandler::get_self() {
    return self_.lock();
  }

  //
  // Build a kernelhandler.
  //
  shared_ptr<kernelhandler>
  kernelhandler::construct(const shared_ptr<kernel> &the_kernel,
                           const shared_ptr<nodeinterface> &deleg)
  {
    shared_ptr<kernelhandler> retval(new kernelhandler(the_kernel, deleg));
    retval->set_self(retval);
    return retval;
  }

  //
  // Goodbye
  //
  kernelhandler::~kernelhandler() throw() {}

  //
  // Complete the request.
  //
//   void kernelhandler::handle(const timestamp &completion_time) {
//     kernel_->start(completion_time, delegate_);
//   }

} // end of namespace sstmac
