/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_KERNELHANDLER_H
#define SSTMAC_SIMULATOR_KERNELHANDLER_H

#include <sstmac/simulator/eventhandler.h>
#include <sstmac/simulator/kernel.h>
#include <boost/shared_ptr.hpp>
#include <boost/weak_ptr.hpp>

namespace sstmac {

  // Important types.
  class kernel;
  class nodeinterface;

  /**
   * Event handler that fires off a kernel.
   */
  class kernelhandler : public eventhandler {
  private:
    boost::shared_ptr<kernel> kernel_;
    boost::shared_ptr<nodeinterface> delegate_;

    boost::weak_ptr<kernelhandler> self_;

  protected:
    /// Hello
    kernelhandler(const boost::shared_ptr<kernel> &the_kernel,
                  const boost::shared_ptr<nodeinterface> &the_delegate_);

    /// Who am I?
    void set_self(const boost::shared_ptr<kernelhandler> &self);

    /// This is who I am.
    boost::shared_ptr<kernelhandler> get_self();

  public:
    /// Build a kernelhandler.
    static boost::shared_ptr<kernelhandler>
    construct(const boost::shared_ptr<kernel> &the_kernel,
              const boost::shared_ptr<nodeinterface> &deleg);

    /// Goodbye
    virtual ~kernelhandler() throw();

    /// Complete the request.
    void handle(const timestamp &completion_time) {
      kernel_->start(completion_time, delegate_);
    }
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_SIMULATOR_KERNELHANDLER_H
