/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_LAUNCHER_H
#define SSTMAC_SIMULATOR_LAUNCHER_H

#include <string>
#include <boost/shared_ptr.hpp>
#include <boost/weak_ptr.hpp>

namespace sstmac {

  // Forward declaration of important types.
  class timestamp;
  class environment;

  /**
   * A launcher that can be cooperatively scheduled by a very naive scheduler.
   */
  class launcher {
  public:
    /// Hasta la vista.
    virtual ~launcher() throw() {
    }

    /// A string description for this type.
    /// Used to output debugging and tracing information.
    virtual const std::string& description() const = 0;

    /// Run this launcher.
    /// \param current_time The simulation time at which this job was started
    /// \param my_world Information regarding the current job, including
    ///        the parallel process id (of type ppid), the sub-processes
    ///        to be run on this node (allowing shared resources),
    ///        and the list of all other nodes involved in the run.
    ///        The lifetime of this object is managed outside the process,
    ///        creating persistent shared_ptr instances to my_world from within
    ///        the process may result in cyclic reference ownership.
    virtual void run(const timestamp &current_time,
                     const boost::shared_ptr<environment> &my_world) = 0;

    /// Test whether this launcher has completed.
    virtual bool completed() const = 0;
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SIMULATOR_LAUNCHER_H
