/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_NETWORKDATA_H
#define SSTMAC_SIMULATOR_NETWORKDATA_H

#include <sstmac/simulator/ppid.h>
#include <sstmac/simulator/taskid.h>
#include <sstmac/simulator/timestamp.h>
#include <boost/shared_ptr.hpp>
#include <stdint.h>

namespace sstmac {

  /**
   * Base type for payload that gets sent over a network.
   */
  class networkdata {
  public:
    virtual ~networkdata() throw() {}

    /// Get another object that corresponds to the receiving end.
    /// Any information that needs to be switched (e.g. if you have any kind of
    /// "me" information) should be appropriately rearranged.
    virtual boost::shared_ptr<networkdata> complement() const = 0;

    /// The size of the transfer in bytes, including any envelope information.
    /// This value is only used to calculate transfer times, it does not need to
    /// correspond to actual object sizes.
    virtual int64_t bytes() const = 0;

    /// The source task.
    virtual const taskid &source_task() const = 0;

    /// The destination task.
    virtual const taskid &dest_task() const = 0;
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SIMULATOR_NETWORKDATA_H
