/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_NETWORKINTERFACE_H
#define SSTMAC_SIMULATOR_NETWORKINTERFACE_H

#include <sstmac/simulator/timestamp.h>
#include <sstmac/simulator/nodeid.h>
#include <sstmac/simulator/ppid.h>
#include <boost/shared_ptr.hpp>

namespace sstmac {

  // Important types.
  class networkdata;
  class eventhandler;

  /**
   * A networkinterface is a delegate between a node and a server module.
   * This object helps ornament network operations with information about
   * the process (ppid) involved.
   */
  class networkinterface {
  public:
    /// Goodbye.
    virtual ~networkinterface() {}
    
    /// Get the ppid associated with this interface.
    virtual const ppid& get_ppid() const = 0;

    /// Schedule a send operation to happen at the given time.
    /// \param start_time is the time when the send is started.
    /// \param recver is the global index of the receiver node.
    /// \param payload is the data sent over the network
    /// \param completion is the eventhandler signaled when the send is complete
    virtual void send(const timestamp &start_time,
                      nodeid recver,
                      const boost::shared_ptr<networkdata> &payload,
                      const boost::shared_ptr<eventhandler> &completion) = 0;
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_SIMULATOR_NETWORKINTERFACE_H
