/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_NODELIST_H
#define SSTMAC_SIMULATOR_NODELIST_H

#include <sstmac/simulator/nodeid.h>
#include <boost/shared_ptr.hpp>
#include <vector>

namespace sstmac {

  // Important types.
  class node;

  /**
   * A shared mapping from nodeid indices onto node pointers.
   */
  class nodelist {
    /// My nodes.
    std::vector< boost::shared_ptr<node> > nodes_;

  protected:
    /// Hi there.
    nodelist();

  public:
    /// Welcome to the new tomorrow.
    static boost::shared_ptr<nodelist> construct();

    /// The number of nodes in the list.
    nodeid size() const;

    /// Get the given node in the list.
    boost::shared_ptr<node> get(const nodeid &id) const;

    /// Add the given number of nodes to the list using the
    /// given object as a template.
    /// \return the index of the new node.
    nodeid add(const boost::shared_ptr<node> &the_node);
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SIMULATOR_NODELIST_H
