/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_OVERCOMMIT_FLOWER_H
#define SSTMAC_SIMULATOR_OVERCOMMIT_FLOWER_H

#include <sstmac/simulator/flower.h>
#include <sstmac/simulator/timestamp.h>
#include <sstmac/network/network.h>
#include <vector>
#include <map>
#include <set>
#include <string>
#include <boost/shared_ptr.hpp>

namespace sstmac {
  
  /**
   * This flower specialization does not model congestion in any way.
   */
  class overcommit_flower: public flower {

    double node_router_bw_;
    double router_router_bw_;
    flow_id_t fid_;
    std::multimap<timestamp, flow_id_t> stop_time_;
    timestamp t_;

    void require_one_flow();

  protected:
    overcommit_flower(const boost::shared_ptr<sstmac::network> &net,
           double node_router_bw, double router_router_bw);

  public:
    /// Construct an overcommit_flower.
    /// \param node_router_bw maximum bandwidth on network links each
    ///        node and router
    /// \param router_router_bw maximum bandwidth on network links each
    ///        router
    /// \return a shared_ptr containing the new object
    static boost::shared_ptr<overcommit_flower>
    construct(const boost::shared_ptr<sstmac::network> &net,
              double node_router_bw, double router_router_bw);

    virtual ~overcommit_flower() throw();

    /// Override flower::start_flow.
    flow_id_t start_flow(timestamp tstart, const std::vector<edge_id_t> &edges, long nbyte);

    /// Override flower::stop_flow.
    void stop_flow(timestamp t, flow_id_t f);

    /// Override flower::next_stopping_time.
    timestamp next_stopping_time();

    /// Override flower::next_stopping_flow.
    flow_id_t next_stopping_flow();
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SIMULATOR_OVERCOMMIT_FLOWER_H
