/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_PPID_H
#define SSTMAC_SIMULATOR_PPID_H

#include <iosfwd>

namespace sstmac {
  class ppid {
  private:
    int id_;
  public:
    explicit ppid(int v = -1) : id_(v) {}
    ppid& operator++() {
      ++id_;
      return *this;
    }
    ppid operator++(int) {
      ppid other(*this);
      this->operator++();
      return other;
    }
    bool operator<(const ppid &other) const {
      return id_ < other.id_;
    }
    int id() const { return id_; }
  };

  std::ostream& operator<<(std::ostream& os, const ppid &p);

} // end of namespace sstmac

#endif // ! SSTMAC_SIMULATOR_PPID_H
