/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/simulator/roundrobinindexing.h>
#include <sstmac/util/errors.h>

namespace sstmac {

  using boost::shared_ptr;

  roundrobinindexing::roundrobinindexing() {
  }

  shared_ptr<roundrobinindexing> roundrobinindexing::construct() {
    return shared_ptr<roundrobinindexing>(new roundrobinindexing);
  }

  roundrobinindexing::~roundrobinindexing() throw() {
  }

  void roundrobinindexing::allocate(const std::vector<nodeid> &nodes,
                                    int ppn,
                                    std::vector<nodeid> &result)
  {
    if(nodes.empty()) {
      throw valueerror("roundrobinindexing::allocate:  Empty node vector.");
    }
    if(ppn <= 0) {
      throw valueerror("roundrobinindexing::allocate:  ppn must be > 0.");
    }
    result = nodes;
    for(int i = 1; i < ppn; ++i) {
      result.insert(result.end(), nodes.begin(), nodes.end());
    }
  }

} // end of namespace sstmac
