/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_SHAREDCIRCUIT_STARTER_H
#define SSTMAC_SIMULATOR_SHAREDCIRCUIT_STARTER_H

#include <sstmac/simulator/sharedcircuit-stopper.h>
#include <vector>

namespace sstmac {

  /**
   * Handle firing off a transfer request for sharedcircuit.
   */
  class sharedcircuit::starter : public eventhandler {
    boost::shared_ptr<flower> flower_;
    boost::shared_ptr<stopper> stopper_;
    int64_t bytes_;
    timestamp recv_delay_;
    std::vector<edge_id_t> edge_id_;
    boost::shared_ptr<eventhandler> sender_;
    boost::shared_ptr<eventhandler> recver_;

  protected:
    /// A new beginning.
    starter(const boost::shared_ptr<flower> &flow,
            const boost::shared_ptr<sharedcircuit::stopper> &stop,
            int64_t bytes,
            const timestamp &recv_delay,
            const std::vector<edge_id_t> edge_id,
            const boost::shared_ptr<eventhandler> &sender,
            const boost::shared_ptr<eventhandler> &recver) :
      flower_(flow), stopper_(stop), bytes_(bytes), recv_delay_(recv_delay),
      edge_id_(edge_id), sender_(sender), recver_(recver)
    {}

  public:
    /// Start me up (kachunk boom).
    static boost::shared_ptr<starter>
    construct(const boost::shared_ptr<flower> &flow,
              const boost::shared_ptr<sharedcircuit::stopper> &stop,
              int64_t bytes,
              const timestamp &recv_delay,
              const std::vector<edge_id_t> edge_id,
              const boost::shared_ptr<eventhandler> &sender,
              const boost::shared_ptr<eventhandler> &recver)
    {
      return boost::shared_ptr<starter>
        (new starter(flow, stop, bytes, recv_delay, edge_id, sender, recver));
    }

    /// I'll never stop.
    virtual ~starter() throw() {}

    /// Our time has come.
    void handle(const timestamp &now) {
      flow_id_t fid = flower_->start_flow(now, edge_id_, bytes_);
      stopper_->add(fid, recv_delay_, sender_, recver_);
      flow_id_t stopflow = flower_->next_stopping_flow();
      timestamp stoptime = flower_->next_stopping_time();
      stopper_->update(stopflow, stoptime);
    }
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SIMULATOR_SHAREDCIRCUIT_STARTER_H
