/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/simulator/simplenode-netcall.h>
#include <sstmac/util/errors.h>

namespace sstmac {

  // Pointer types.
  using boost::shared_ptr;
  using boost::weak_ptr;

  //
  // Hi.
  //
  simplenode::netcall::netcall(const shared_ptr<simplenode> &parent, const ppid &pid) :
    parent_(parent), pid_(pid)
  {}

  //
  // Hello.
  //
  shared_ptr<simplenode::netcall>
  simplenode::netcall::construct(const shared_ptr<simplenode> &parent, const ppid &pid)
  {
    return shared_ptr<netcall>(new netcall(parent, pid));
  }

  //
  // Goodbye.
  //
  simplenode::netcall::~netcall() throw() {
  }

  //
  // Get the ppid associated with this interface.
  //
  const ppid& simplenode::netcall::get_ppid() const {
    return pid_;
  }

  //
  // Schedule a send operation to happen at the given time.
  //
  void simplenode::netcall::send(const timestamp &start_time,
                                 nodeid recver,
                                 const boost::shared_ptr<networkdata> &payload,
                                 const boost::shared_ptr<eventhandler> &completion)
  {
    parent_.lock()->send(start_time, pid_, recver, payload, completion);
  }

} // end of namespace sstmac.
