/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_SIMPLENODE_NETCALL_H
#define SSTMAC_SIMULATOR_SIMPLENODE_NETCALL_H

#include <sstmac/simulator/simplenode.h>
#include <sstmac/simulator/networkinterface.h>

namespace sstmac {

  /**
   * A private nested type to handle sending data.
   */
  class simplenode::netcall : public networkinterface {
    boost::weak_ptr<simplenode> parent_;
    ppid pid_;
  protected:
    /// Hi.
    netcall(const boost::shared_ptr<simplenode> &parent, const ppid &pid);

  public:
    /// Hello.
    static boost::shared_ptr<netcall>
    construct(const boost::shared_ptr<simplenode> &parent, const ppid &pid);

    /// Goodbye.
    virtual ~netcall() throw();

    /// Get the ppid associated with this interface.
    virtual const ppid& get_ppid() const;

    /// Schedule a send operation to happen at the given time.
    /// \param start_time is the time when the send is started.
    /// \param recver is the global index of the receiver node.
    /// \param payload is the data sent over the network
    /// \param completion is the eventhandler signaled when the send is complete
    virtual void send(const timestamp &start_time,
                      nodeid recver,
                      const boost::shared_ptr<networkdata> &payload,
                      const boost::shared_ptr<eventhandler> &completion);
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_SIMULATOR_SIMPLENODE_NETCALL_H
