/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SKELETON_DUMMY_DUMMYKERNEL_H
#define SSTMAC_SKELETON_DUMMY_DUMMYKERNEL_H

#include <sstmac/skeleton/dummy/dummyserver.h>
#include <sstmac/simulator/kernel.h>
#include <sstmac/simulator/eventhandler.h>
#include <sstmac/simulator/tasklist.h>
#include <sstmac/simulator/nodeinterface.h>
#include <sstmac/util/errors.h>
#include <boost/weak_ptr.hpp>
#include <iostream>

namespace sstmac {

  class dummykernel : public kernel {
    /// A private nested type to handle completion of ping-pong events.
    class notify : public eventhandler {
      timestamp start_;
      taskid me_, dest_;
    protected:
      notify(const timestamp &start, const taskid &me, const taskid &dest) :
        start_(start), me_(me), dest_(dest)
      {}
    public:
      static boost::shared_ptr<notify>
      construct(const timestamp &start, const taskid &me, const taskid &dest) {
        return boost::shared_ptr<notify>(new notify(start, me, dest));
      }
      virtual ~notify() throw() {}
      void handle(const timestamp &now) {
        //std::cerr << "Node " << me_ << " pinged " << dest_
        //          << ".  Roundtrip time=" << (now-start_) << "\n";
      }
    };

    /// Private members of class dummykernel
    boost::shared_ptr<tasklist> tlist_;
    boost::shared_ptr<dummyserver> server_;
  protected:
    /// Build us a kernel.
    dummykernel(const boost::shared_ptr<tasklist>& tlist,
                const boost::shared_ptr<dummyserver>& server) :
      tlist_(tlist), server_(server)
    {}

  public:
    /// Welcome to the dummy world of tomorrow.
    static boost::shared_ptr<dummykernel>
    construct(const boost::shared_ptr<tasklist>& tlist,
              const boost::shared_ptr<dummyserver>& server,
              const boost::shared_ptr<eventhandler>& completion)
    {
      boost::shared_ptr<dummykernel> retval(new dummykernel(tlist, server));
      retval->add_listener(completion);
      return retval;
    }

    /// Thank you for choosing the dummykernel suicide booth.
    /// You are now dead.
    virtual ~dummykernel() throw() {}
    
    /// And here we go.
    virtual void start(const timestamp &start_time,
                       const boost::shared_ptr<nodeinterface>& node)
    {
      std::cout << "dummykernel started at " << start_time << "\n";
      taskid me = node->get_taskid();
      taskid target((me.id + 1) % tlist_->size().id);
      server_->ping(me, target, notify::construct(start_time, me, target));
      kernel::complete(start_time);
    }
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_SKELETON_DUMMY_DUMMYKERNEL_H
