/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SKELETON_MINIMD_MINIMD_INTEGRATE_H
#define SSTMAC_SKELETON_MINIMD_MINIMD_INTEGRATE_H

#include <sstmac/skeleton/minimd/minimd.h>
#include <sstmac/skeleton/minimd/minimd-comm.h>
#include <sstmac/skeleton/minimd/minimd-timer.h>
#include <sstmac/skeleton/minimd/minimd-force.h>
#include <sstmac/skeleton/minimd/minimd-thermo.h>
#include <sstmac/skeleton/minimd/minimd-neighbor.h>

namespace sstmac {

  using namespace boost;

  /**
   * Take over the roles of the time integrator in miniMD.
   */
  class minimd::integrate {
  private:
    shared_ptr<runinterpolator> execution_;
    shared_ptr<mpiapi> mpi_;

  public:
    double dt;
    int ntimes;
    
    void init(const shared_ptr<runinterpolator>& exe,
              const shared_ptr<mpiapi>& mpi)
    {
      execution_ = exe;
      mpi_ = mpi;
    }

    void run(const shared_ptr<atom>& atm, const shared_ptr<force>& frc,
             const shared_ptr<neighbor>& nbr, const shared_ptr<comm>& cmm,
             const shared_ptr<thermo>& thm, const shared_ptr<timer>& tmr)
    {
      mpiid rank = mpi_->comm_world().rank();
//#define SSTMAC_DB(command) std::cerr << rank << " " << #command << "\n"; command;
#define SSTMAC_DB(command) command
      for(int n = 0; n < ntimes; ++n) {
        SSTMAC_DB( mpi_->compute(execution_->get("Integrate::run", 0, 1)) );

        if((n+1) % nbr->every) {
          SSTMAC_DB( cmm->communicate(atm) );
          SSTMAC_DB( tmr->stamp(timer::TIME_COMM) );
        }
        else {
          SSTMAC_DB( cmm->exchange(atm) );
          SSTMAC_DB( cmm->borders(atm) );
          SSTMAC_DB( tmr->stamp(timer::TIME_COMM) );
          SSTMAC_DB( nbr->build(atm) );
          SSTMAC_DB( tmr->stamp(timer::TIME_NEIGH) );
        }
        
        SSTMAC_DB( frc->compute(atm, nbr) );
        SSTMAC_DB( tmr->stamp(timer::TIME_FORCE) );
        
        SSTMAC_DB( mpi_->compute(execution_->get("Integrate::run", 0, 2)) );
        if(thm->nstat) {
          SSTMAC_DB( thm->compute(n+1, atm, nbr, frc) );
        }
      }
#undef SSTMAC_DB
    }
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SKELETON_MINIMD_MINIMD_INTEGRATE_H
