/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SKELETON_MINIMD_MINIMD_THERMO_H
#define SSTMAC_SKELETON_MINIMD_MINIMD_THERMO_H

#include <sstmac/skeleton/minimd/minimd.h>
#include <sstmac/skeleton/minimd/minimd-force.h>

namespace sstmac {

  using boost::shared_ptr;

  /**
   * Pretend to do thermodynamics.
   */
  class minimd::thermo {
  private:
    boost::shared_ptr<runinterpolator> execution_;
    boost::shared_ptr<mpiapi> mpi_;

  public:
    int nstat;
    int ntimes;

    void init(const boost::shared_ptr<runinterpolator>& exe,
              const boost::shared_ptr<mpiapi>& mpi)
    {
      execution_ = exe;
      mpi_ = mpi;
    }

    void compute(int iflag, const shared_ptr<atom>& atm, const shared_ptr<neighbor>& nbr,
                 const shared_ptr<force>& frc)
    {
      if (iflag > 0 && iflag % nstat) return;
      if (iflag == -1 && nstat > 0 && ntimes % nstat == 0) return;      

      temperature(atm);
      energy(atm);
      frc->compute(atm, nbr);
      pressure(atm);
      mpi_->compute(execution_->get("Thermo::compute"));
    }
    
    void energy(const shared_ptr<atom>&) {
      mpi_->compute(execution_->get("Thermo::energy"));
      mpi_->allreduce(1, mpitype::mpi_double, mpiop::sum, mpi_->comm_world());
    }

    void temperature(const shared_ptr<atom>&) {
      mpi_->compute(execution_->get("Thermo::temperature"));
      mpi_->allreduce(1, mpitype::mpi_double, mpiop::sum, mpi_->comm_world());
    }

    void pressure(const shared_ptr<atom>&) {
      mpi_->compute(execution_->get("Thermo::pressure"));
      mpi_->allreduce(1, mpitype::mpi_double, mpiop::sum, mpi_->comm_world()); 
    }
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SKELETON_MINIMD_MINIMD_THERMO_H
