/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SKELETON_MINIMD_MINIMD_H
#define SSTMAC_SKELETON_MINIMD_MINIMD_H

#include <sstmac/process/mpiapp.h>
#include <string>

namespace sstmac {

  /// Important types.
  class runinterpolator;

  /**
   * Early pass at turning miniMD into a skeleton application.
   */
  class minimd : public mpiapp {
    /// Nested types mirroring the layout of minimd.
    class in;
    class atom;
    class force;
    class neighbor;
    class integrate;
    class thermo;
    class comm;
    class timer;

    /// The input file name.
    std::string infile_;
    /// The interpolator/lookup table for our execution times.
    boost::shared_ptr<runinterpolator> execution_;
    /// The nested private objects.
    boost::shared_ptr<in> in_;
    boost::shared_ptr<atom> atom_;
    boost::shared_ptr<force> force_;
    boost::shared_ptr<neighbor> neighbor_;
    boost::shared_ptr<integrate> integrate_;
    boost::shared_ptr<thermo> thermo_;
    boost::shared_ptr<comm> comm_;
    boost::shared_ptr<timer> timer_;

  private:
    /// Mirror static functions in mpiMD.
    void input();
    void create_box();
    void create_atoms();
    void create_velocity();
    void output();
    
  protected:
    /// Hi.
    explicit minimd(const std::string &file,
                    const boost::shared_ptr<runinterpolator>& exe);

  public:
    /// Build a new minimd object, which will read its input from
    /// the given file.
    static boost::shared_ptr<minimd>
    construct(const std::string &file,
              const boost::shared_ptr<runinterpolator>& exe);
    
    /// Goodbye.
    virtual ~minimd() throw();

    /// Get a copy of this object.
    boost::shared_ptr<mpiapp> clone();
    
    /// We use init_mpi to initialize a bunch of other stuff as well.
    void init_mpi(const boost::shared_ptr<mpiapi>& mpi);

    /// Go.
    void run();
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SKELETON_MINIMD_MINIMD_H
