/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/skeleton/mpiparsers/parsedumpi.h>
#include <sstmac/skeleton/mpiparsers/parsedumpi-callbacks.h>
#include <sstmac/skeleton/mpiparsers/dumpimeta.h>
#include <sstmac/util/errors.h>
#include <boost/lexical_cast.hpp>
#include <stdio.h>
#include <errno.h>
#include <string.h>

namespace sstmac {

  using boost::shared_ptr;

  //
  // Hi there.
  //
  parsedumpi::parsedumpi(const std::string &fileroot, double timescaling) :
    fileroot_(fileroot), timescaling_(timescaling)
  {}
  
  //
  // Hi there.
  //
  parsedumpi::parsedumpi(const boost::shared_ptr<dumpimeta>& dmeta, double timescaling) :
    timescaling_(timescaling), dmeta_(dmeta)
  {
  	fileroot_ = dmeta_->dirplusfileprefix_;
  }

  //
  // Good news everyone!  We are getting a new parsedumpi.
  //
  shared_ptr<parsedumpi>
  parsedumpi::construct(const std::string &fileroot, double timescaling) {
    shared_ptr<parsedumpi> retval(new parsedumpi(fileroot, timescaling));
    retval->set_self(retval);
    return retval;
  }
  
  //
  // Good news everyone!  We are getting a new parsedumpi.
  //
  shared_ptr<parsedumpi>
  parsedumpi::construct(const shared_ptr<dumpimeta>& dmeta,
                        double timescaling)
  {
    shared_ptr<parsedumpi> retval(new parsedumpi(dmeta, timescaling));
    retval->set_self(retval);
    return retval;
  }

  //
  // Wait!  That's not good news at all!
  //
  parsedumpi::~parsedumpi() throw() {
  }

  //
  // IT'S AN ADDAMS!
  //
  shared_ptr<mpiapp> parsedumpi::clone() {
    // put if to check fileroot
    return construct(fileroot_, timescaling_);
  }

  //
  // Parse the file please (this needs an update, there is no reason
  // for it to return a TRACEREC (which should not be uppercase).
  // Need to figure out how to give the caller access to header/footer recs.
  //
  void parsedumpi::run() {
    static const std::string here("parsedumpi::getmessage");
    if((mpi() == 0) || (world().rank().id < 0))
      throw illformederror(here + ":  Dumpi parser not initialized.");
    callbacks cbacks(this);
    // Figure out our input file name.
    // Try fileroot-"%01d", fileroot-"%02d", ... up to some reasonable length
    // (in the short term that we will probably not have over 10 billion peers)
    int rank = world().rank().id;
    FILE *fp = NULL;
    char fname[256]; fname[255] = '\0';
    const char *fnamefmt = "%%s-%%0%dd.bin";
    char fmt[30]; fmt[29] = '\0';
    for(int i = 1; i < 10; ++i) {
      snprintf(fmt, 29, fnamefmt, i);
      snprintf(fname, 255, fmt, fileroot_.c_str(), rank);
      //std::cout << "parsedumpi::getmessage(): looking for file " << fname << std::endl;
      fp = fopen(fname, "r");
      if(fp) {
        fclose(fp);
        break;
      }
    }
    // If we get here with a null FILE pointer, no file was found.
    if(fp == NULL) {
      throw ioerror(here + " failed to find an input file starting from " +
		    "the fileroot \"" + fileroot_ + "\": " + strerror(errno));
    }
    else {
      // Ready to go.
      cbacks.parse_stream(fname);
    }
  }

} // end of namespace sstmac

