/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SKELETON_MPIPARSERS_PARSEDUMPI_H
#define SSTMAC_SKELETON_MPIPARSERS_PARSEDUMPI_H

#include <sstmac/process/mpiapp.h>
#include <sstmac/process/mpiapi.h>
#include <string>
#include <vector>
#include <map>
#include <boost/shared_ptr.hpp>
#include <boost/lexical_cast.hpp>

namespace sstmac {

  /// Important types.
  class dumpimeta;

  /**
   * A refactored dumpi parser to read the newer binary-format dumpi
   * trace files.  Discards support for older-style ascii-based
   * files since nobody was using those except us anyway.
   *
   * TODO:  Fix this to read the metafile rather than the trace files directly
   * (would avoid hack needed where MPI world size is smaller than the total
   * cores allocated to the job).
   */
  class parsedumpi : public mpiapp {
    /// Private parts:
    class callbacks;

    /// The fileroot we plan to parse.
    std::string fileroot_;

    /// The time scaling factor.
    double timescaling_;
    
    /// The metafile object
    boost::shared_ptr<dumpimeta> dmeta_;

    /// Convenience method.
    inline const mpicomm& world() {
      return mpi()->comm_world();
    }

  protected:
    /// Hi there.
    parsedumpi(const std::string &fileroot, double timescaling);
    
    /// Hi there II. 
    parsedumpi(const boost::shared_ptr<dumpimeta>& dmeta, double timescaling);

  public:
    /// Good news everyone!  We are getting a new parsedumpi.
    static boost::shared_ptr<parsedumpi>
    construct(const std::string &fileroot, double timescaling);
    
    /// Better news everyone!  We are getting a new parsedumpi with meta.
    static boost::shared_ptr<parsedumpi>
    construct(const boost::shared_ptr<dumpimeta>& dmeta, double timescaling);

    /// Wait!  That's not good news at all!
    virtual ~parsedumpi() throw();

    /// IT'S AN ADDAMS!
    virtual boost::shared_ptr<mpiapp> clone();

    /// Parse the tracefile.
    virtual void run();
  };


} // end of namespace sstmac

#endif // ! SSTMAC_SKELETON_MPIPARSERS_PARSEDUMPI_H
