/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/skeleton/mpiparsers/rsqmetatrace.h>
#include <sstmac/util/errors.h>
#include <algorithm>

namespace sstmac {

  using boost::shared_ptr;
  
  //
  // Create a new metatrace with enough space for the given number of peers.
  //
  rsqmetatrace::rsqmetatrace(const std::string &fileroot, size_t peers) :
    fileroot_(fileroot), peer_crd_(peers)
  {}

  //
  // Hello.
  //
  shared_ptr<rsqmetatrace>
  rsqmetatrace::construct(const std::string &fileroot, size_t peers) {
    return shared_ptr<rsqmetatrace>(new rsqmetatrace(fileroot, peers));
  }

  //
  // Farewell.
  //
  rsqmetatrace::~rsqmetatrace() throw() {
  }

  //
  // The file root.
  //
  const std::string& rsqmetatrace::fileroot() const {
    return fileroot_;
  }

  //
  // The number of nodes in the system.
  //
  nodeid rsqmetatrace::nodes() const {
    //std::cout << "rsqmetatrace::nodes() returning " << node_crd_.size() << std::endl;
    return nodeid(node_crd_.size());
  }

  //
  // Get the coordinate of the nth node in the run.
  //
  const rsqcrd& rsqmetatrace::node_crd(const nodeid &node) const {
    return node_crd_.at(node.id);
  }

  //
  // The number of peers in this trace.
  //
  mpiid rsqmetatrace::peers() const {
    //std::cout << "rsqmetatrace::peers() returning " << peer_crd_.size() << std::endl;
    return mpiid(peer_crd_.size());
  }

  //
  // Get the coordinate of the nth peer in the run
  //
  const rsqcrd& rsqmetatrace::peer_crd(const mpiid &peer) const {
    return peer_crd_.at(peer.id);
  }

  //
  // Set the mesh coordinate of the nth node.
  //
  void rsqmetatrace::set_peer_crd(const mpiid &peer, const rsqcrd &crd) {
    peer_crd_.at(peer.id) = crd;
    if(std::find(node_crd_.begin(), node_crd_.end(), crd) == node_crd_.end())
      node_crd_.push_back(crd);
  }

} // end of namespace sstmac
