/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_UTIL_BUILDER_H
#define SSTMAC_UTIL_BUILDER_H

#include <string>
#include <typeinfo>
#include <boost/shared_ptr.hpp>

namespace sstmac {

  /**
   * Base type for builder objects.  Used to ensure that we can
   * retrieve a builder based on a string identifier from the
   * builderrepo singleton.  Creation of a builder object
   * is automatically registered with the builderrepo.
   */
  class builder {
  protected:
    /// Create a new builder object and register it by name in the repo.
    builder() {
    }

  public:
    /// Goodbye cruel world.
    virtual ~builder() {
    }

    /// The label associatied with this builder.
    /// This label gets used to store the builder in the repository.
    virtual const std::string& label() const = 0;

    /// The type_info of the concrete type generated by this builder.
    virtual const std::type_info& type() const = 0;
  };

} // end of namespace sstmac

#endif // ! SSTMAC_UTIL_BUILDER_H
