/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_UTIL_ERRORS_H
#define SSTMAC_UTIL_ERRORS_H

#include <string>
#include <exception>
#include <stdlib.h> // abort()

namespace sstmac {

  struct ssterror : public std::exception {
    ssterror(const std::string &msg) : message(msg) {}
    virtual ~ssterror() throw() {}
    virtual const char* what() const throw() { return message.c_str(); }
    const std::string message;    
  };

  struct nullerror : public ssterror {
    nullerror(const std::string &msg) : ssterror(msg) {}
    virtual ~nullerror() throw() {}
  };

  struct valueerror : public ssterror {
    valueerror(const std::string &msg) : ssterror(msg) {}
    virtual ~valueerror() throw() {}
  };

  struct timeerror : public ssterror {
    timeerror(const std::string &msg) : ssterror(msg) {}
    virtual ~timeerror() throw() {}
  };

  struct ioerror : public ssterror {
    ioerror(const std::string &msg) : ssterror(msg) {}
    virtual ~ioerror() throw() {}
  };

  struct illformederror : public ssterror {
    illformederror(const std::string &msg) : ssterror(msg) {}
    virtual ~illformederror() throw() {}
  };

  struct schedulererror : public ssterror {
    schedulererror(const std::string &msg) : ssterror(msg) {}
    virtual ~schedulererror() throw() {}
  };

  struct memoryerror : public ssterror {
    memoryerror(const std::string &msg) : ssterror(msg) {}
    virtual ~memoryerror() throw() {}
  };

  struct iteratorerror : public ssterror {
    iteratorerror(const std::string &msg) : ssterror(msg) {}
    virtual ~iteratorerror() throw() {}
  };

  struct unimplementederror : public ssterror {
    unimplementederror(const std::string &msg) : ssterror(msg) {}
    virtual ~unimplementederror() throw() {}
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_UTIL_ERRORS_H
