/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_UTIL_SERIALIZABLE_H
#define SSTMAC_UTIL_SERIALIZABLE_H

#include <boost/shared_ptr.hpp>

namespace sstmac {

  // Forward declaration of relevant types.
  class streamin;
  class streamout;

  /**
   * Base type for objects that can be serialized to and from a
   * stream object.
   *
   * Concrete derived types must be registered by 
   * (1) defining a (static) pointer to a dynamically allocated
   * serialbuilder instance in a source (.cc) file, and
   * (2) have a constructor that takes a shared_ptr<stream>
   * as an input argument.
   */
  class serializable {
  public:
    serializable();

    /// Derived types should call this constructor when rebuilding
    /// from a streamin object.
    /// \throw nullerror if the stream object is NULL.
    serializable(boost::shared_ptr<streamin>);

    virtual ~serializable();

    /// Save data to the given streamout object.
    virtual void write_data(streamout*) = 0;
  };

} // end of namespace sstmac

#endif // ! SSTMAC_UTIL_SERIALIZABLE_H
