/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_UI_SIMTABS_H
#define SSTMAC_UI_SIMTABS_H

#include <sstmac/simulator/manager.h>
#include <sstmac/simulator/circuit.h>
#include <iostream>
#include <fstream>
#include <boost/shared_ptr.hpp>

#include <QDialog>
#include <QMainWindow>
#include <QtGui/QApplication>

QT_BEGIN_NAMESPACE
class QDialogButtonBox;
class QTabWidget;
class QMenu;
class QMenuBar;
class QGroupBox;
class QLabel;
class QLineEdit;
class QRadioButton;
class QComboBox;
//mdi
class QMdiArea;
class QMdiSubWindow;
class QSignalMapper;
QT_END_NAMESPACE

using boost::shared_ptr;
using namespace sstmac;

class runmetrics;
typedef boost::shared_ptr<runmetrics> RMPtr;

class qapplaunch : public QApplication {

 public:
  qapplaunch(int &argc, char *argv[]);
  bool notify(QObject * receiver, QEvent * event);
};

class SimDialog : public QMainWindow
{
  Q_OBJECT

 public:
  SimDialog( QWidget *parent = 0);
       
 private slots:
  void closethetab(int i);
  void customcascadeSubWindows();
  void updateWindowMenu();
  void updateMenus();
  void setActiveSubWindow(QWidget *window);
  void closeandcheck();
  void closeallandcheck();

 private:
  QTabWidget *tabWidget;
  QDialogButtonBox *buttonBox;
  //qapplaunch *appl_;
        
  //mdi
  QMdiArea *mdiArea;
  QSignalMapper *windowMapper;
        
  //gui
  QMenu *fileMenu;
  QMenu *windowMenu;
  QAction *exitAct;
  QAction *closeAct;
  QAction *closeAllAct;
  QAction *tileAct;
  QAction *cascadeAct;
  QAction *separatorAct;
  void createMenus();
  QWidget *activeMdiChild();
  QMdiSubWindow *findMdiChild();
};

class runmetrics
{
 public:
  runmetrics(double time, char *metafilename);
  runmetrics& operator=(const runmetrics& that);
                
  double time_;
  char *metafilename_;
};

class resultstab : public QWidget
{
  Q_OBJECT
    
 public:
  resultstab(RMPtr metrics, QWidget *parent = 0);
  void tokenize(const std::string& str, std::vector<std::string>& tokens, const std::string& delimiters );
   
 private:
  RMPtr metrics_;
};

class parametertab : public QWidget
{
  Q_OBJECT

 public:
  parametertab(QTabWidget *tW, QMdiArea *ma, QWidget *parent = 0);
  bool setupsim();
  void test();
  QWidget *child;
  QMainWindow *mainWin;
    
 private slots:
  void enableFields(int);
  void setOpenFileName();
  void runsim();
  void genpythonscript();
  void topologyChanged(int);
  void typeChanged(int);
  //qt
  void setActiveSubWindow(QWidget *window);
                
 private:
                
  enum NETTYPE { CROSSBAR, FATTREE, GAMMA, MESH2D, MESH3D, RING, TORUS2D, TORUS3D, TORUS4D };
  RMPtr rmetrics_;
  std::fstream fs;
  std::string fstext_;
  char *metafilename_;
  std::streampos text_start_;
                
  QTabWidget *tabWidget;
  QMdiArea *mdiArea;
  QSignalMapper *windowMapper;
                
  QString curFile;
  int sequencenum;
                
  //qt
  QMenu *fileMenu;
  QMenu *windowMenu;
  QAction *closeAct;
  QAction *closeAllAct;
  QAction *tileAct;
  QAction *cascadeAct;
  QAction *separatorAct;
  QAction *exitAct;
  QWidget *activeMdiChild();
                
  bool scriptonly;
  void initpythonscr();
  void createNetworkGroupBox();
  void createParameterGroupBox();
  void createControlGroupBox();
  void createSimTypeGroupBox();
  void createSimGroupBox();
  void createFullIndexGroupBox();
  void createFullAllocationGroupBox();
  void createCircuitIndexGroupBox();
  void createCircuitAllocationGroupBox();
  void createProcsGroupBox();
  void createNodesGroupBox();
  void createOutputLabelGroupBox();
        
  void fullnetview();
  void circuitview();
        
  void turnfieldsoff();
  void turnfieldson(NETTYPE n);
        
  bool validatefields(shared_ptr<interconnect> &fab, shared_ptr<sstmac::network> &net, int &nnodes, int &ppn);
  bool validateFullnetwork(shared_ptr<interconnect> &fab);
  bool validateCrossbar(shared_ptr<sstmac::network> &net);
  bool validateFattree(shared_ptr<sstmac::network> &net);
  bool validateGamma(shared_ptr<sstmac::network> &net);
  bool validateMesh2D(shared_ptr<sstmac::network> &net);
  bool validateMesh3D(shared_ptr<sstmac::network> &net);
  bool validateRing(shared_ptr<sstmac::network> &net);
  bool validateTorus2D(shared_ptr<sstmac::network> &net);
  bool validateTorus3D(shared_ptr<sstmac::network> &net);
  bool validateTorus4D(shared_ptr<sstmac::network> &net);
  bool validateLatBand(shared_ptr<interconnect> &fab, shared_ptr<sstmac::network> &net);
  bool validateint(QString text, int min, int max);
  bool validatedouble(QString text, double min, double max, double dec);
  bool validateN1andN2();
  bool validateN1();
  bool validateN3();
  bool validateNx();
  bool validatewidth();
  bool validateswitch();
  bool validateDegree();
  bool validateDiameter();
  bool validateLinks();
  bool validateLevels();
  bool validatennodes(int &nnodes);
  bool validateppn(int &ppn);
  bool validatepingpong(int &iter, int &arrlen);
  bool validatetracedriven(std::string &fname, double &tscale);
  bool validateMemBand();
  bool validateFullMemBand();
  bool validateLatBandMem(timestamp &lat, double &band);
                
  void construct_indexing(shared_ptr<sstmac::indexstrategy> &ind);
  void construct_allocation(shared_ptr<sstmac::allocationstrategy> &allocation);
  void construct_manager(shared_ptr<manager> &mgr, shared_ptr<interconnect> &fab,int nnodes);
    
  QMenuBar *menuBar;
  QWidget *networkGroupBox;
  QWidget *fullGroupBox;
  QTabWidget *interTabWidget;
        
  QGroupBox *parameterGroupBox;
  QGroupBox *controlGroupBox;
  QGroupBox *fileGroupBox;
  QGroupBox *simGroupBox;
  QGroupBox *indexGroupBox;
  QGroupBox *allocationGroupBox;
  QGroupBox *procsGroupBox;
  QGroupBox *nodesGroupBox;
  QGroupBox *outputGroupBox;

  QAction *exitAction;
        
  //network group
  QRadioButton *radiocircuit;
  QRadioButton *radiofullnet;
        
  QComboBox *interconnectComboBox;
  QComboBox *topologyComboBox;
  QComboBox *indexComboBox;
  QComboBox *allocationComboBox;
  QComboBox *indexCircuitComboBox;
  QComboBox *allocationCircuitComboBox;
        
  QLabel *labeln1;
  QLabel *labeln2;
  QLabel *labeln3;
  QLabel *labelnx;
  QLineEdit *lineEditn1;
  QLineEdit *lineEditn2;
  QLineEdit *lineEditn3;
  QLineEdit *lineEditnx;
        
  QLabel *labelcrossnnodes;
  QLineEdit *lineEditcrossnnodes;

  QLabel *labelFatLevels;
  QLabel *labelFatLinks;
  QLineEdit *lineEditFatLevels;
  QLineEdit *lineEditFatLinks;
        
  QLabel *labelGammaDegree;
  QLabel *labelGammaDiameter;
  QLineEdit *lineEditGammaDegree;
  QLineEdit *lineEditGammaDiameter;
   
  QLabel *labelnnodeperswitch;
  QLabel *labelwidth;
  QLineEdit *lineEditnnodeperswitch;
  QLineEdit *lineEditwidth;
        
  QLineEdit *lineEditTimeScale;
  QLabel *labelTimeScale;
  QLineEdit *lineeditFileName;
  QPushButton *buttonBrowse;
  QLabel *labelIteration;
  QLabel *labeldatalength;
  QLineEdit *lineEditIter;
  QLineEdit *lineEditPacket;
                
  QLineEdit *lineEditMemBand;
  QLineEdit *lineEditFullMemBand;
  double mem_band;
    
  QLineEdit *lineEditIntraLat;
  QLineEdit *lineEditInterLat;
  QLineEdit *lineEditIntraBand;
  QLineEdit *lineEditInterBand;
  QLineEdit *lineeditnnodes;
  QLineEdit *lineeditprocspernode;
                
  QLabel *labelLatency;
  QLabel *labelBandwidth;
  QLabel *labelIntraLat;
  QLabel *labelInterLat;
  QLabel *labelIntraBand;
  QLabel *labelInterBand;
  QComboBox *comboboxLat;
  QComboBox *comboboxBand;
  QComboBox *comboboxLat1;
  QComboBox *comboboxBand1;
  QComboBox *typeComboBox;
                
  QLabel *labelLat;
  QLabel *labelBand;
  QLineEdit *lineEditLat;
  QLineEdit *lineEditBand;
                
  QLineEdit *lineEditOutput;
};


#endif
