/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/simpleapp.h>
#include <sstmac/process/applaunch.h>
#include <sstmac/process/scheduler.h>
#include <sstmac/simulator/manager.h>
#include <sstmac/simulator/fullnetwork.h>
#include <sstmac/simulator/eventmap.h>
#include <sstmac/simulator/simplenode.h>
#include <sstmac/simulator/taskid.h>
#include <sstmac/processor/computeapi.h>
#include <sstmac/processor/simpleprocessor.h>
#include <sstmac/processor/simpleprocessor-config.h>
#include <sstmac/util/debug.h>
#include <unistd.h> // getopt
#include <string>

using boost::shared_ptr;
using namespace sstmac;

void punt() {
  std::cerr << "Error: usage: testsimpleapp [--config filename]\n"; 
  abort();
}

int main (int argc, char* argv[]) {

  //debug().to_stderr();

  size_t stack = 1<<15;
  int nodes = 2;

  // get config object for the processor model
  boost::shared_ptr<simpleprocessor_config> proc_cfg;
  if(argc==1)
    proc_cfg = simpleprocessor_config::construct("PAPI_FP_INS",timestamp::exact_nsec(10),
                                                 "PAPI_LST_INS",timestamp::exact_nsec(250));
  else if(argc==3) {
    if( !std::string(argv[1]).compare("--config") )
      proc_cfg = simpleprocessor_config::construct( argv[2] );
    else punt();
  }
  else punt();

  shared_ptr<manager> mgr = manager::construct();
  mgr->set_interconnect(fullnetwork::construct(timestamp(1e-9), 125000000));
  mgr->set_eventmanager(eventmap::construct());
  boost::shared_ptr<simplenode> node = simplenode::construct(int(1e10),2);
  boost::shared_ptr<simpleprocessor> proc = 
    simpleprocessor::construct( proc_cfg );
  node->set_processor(proc);
  mgr->add_nodes(node, nodes);

  shared_ptr<applaunch> job(applaunch::construct());
  std::map< taskid,boost::shared_ptr<app> > apps;
  for(int i=0; i<nodes; ++i)
    apps[taskid(i)] = simpleapp::construct();
  job->set_applications(apps);
  job->set_scheduler(scheduler::construct(stack, stack*nodes, false));
  mgr->add_launcher(timestamp(1), job, nodes, 1);
  timestamp t = mgr->run();
  std::cout << "end time is " << t << std::endl;

}

