/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/util/debug.h>
#include <sstmac/process/key.h>
#include <sstmac/skeleton/dummy/dummyserver.h>
#include <sstmac/skeleton/dummy/dummylauncher.h>
#include <sstmac/simulator/manager.h>
#include <sstmac/simulator/fullnetwork.h>
#include <sstmac/simulator/eventmap.h>
#include <sstmac/simulator/simplenode.h>
#include <sstmac/simulator/kernelhandler.h>
#include <sstmac/simulator/ppid.h>
#include <sstmac/simulator/taskid.h>
#include <sstmac/simulator/nodeid.h>
#include <sstmac/simulator/networkcalltrap.h>
#include <sstmac/process/scheduler.h>
#include <sstmac/processor/simpleprocessor.h>
#include <sstmac/processor/simpleprocessor-config.h>
#include <sstmac/processor/eventdata.h>
#include <sstmac/processor/computekernel.h>
#include <unistd.h> // getopt

namespace sstmac {

  using boost::shared_ptr;

  void simpleprocrun() {

    //debug().to_stderr();
    //debug().to_stream(std::cout);
    //debug().to_file("./output");
    boost::shared_ptr<eventmanager> ev_manager = eventmap::construct();

    taskid tid;
    ppid pid;
    nodeid nid;

    // construct a node
    boost::shared_ptr<simplenode> node = 
      simplenode::construct(int(1e10));
    // use a dummy network interface
    boost::shared_ptr<networkcall> network =
      networkcalltrap::construct(boost::shared_ptr<interconnect>(),
                                 boost::shared_ptr<nodelist>(),
                                 nodeid() );
    node->initialize(nid,network,ev_manager);

    // construct processor model and set it on node
    boost::shared_ptr<simpleprocessor> processor = 
      simpleprocessor::construct(simpleprocessor_config::construct());
    node->set_processor(processor);

    // create eventdata objects
    boost::shared_ptr<eventdata> instructions1 = eventdata::construct(); 
      instructions1->set_event("PAPI_FP_INS",1000);
      instructions1->set_event("PAPI_LST_INS",5);
    boost::shared_ptr<eventdata> instructions2 = eventdata::construct();
      instructions2->set_event("PAPI_FP_INS",2000);
      instructions2->set_event("PAPI_LST_INS",100);

    // create computekernel's
    shared_ptr<key> key1 = key::construct();
    boost::shared_ptr<computekernel> kernel1 = 
      computekernel::construct(key1,instructions1);
    shared_ptr<key> key2 = key::construct();
    boost::shared_ptr<computekernel> kernel2 =
      computekernel::construct(key2,instructions2);

    // schedule computekernel
    shared_ptr<eventhandler> handler1(
      kernelhandler::construct(kernel1,node->get_nodeinterface(pid,tid)));
    shared_ptr<eventhandler> handler2(
      kernelhandler::construct(kernel2,node->get_nodeinterface(pid,tid)));
    kernel1->add_listener(handler2);
    ev_manager->schedule( ev_manager->now(), handler1 );

    ev_manager->run();

    std::cout << "time is now " << ev_manager->now() << std::endl;
  }

}

int main(int argc, char **argv) {
  sstmac::simpleprocrun();
}
